@@ -149,7 +149,7 @@ static void testmode(int mode)
     vga_setmode(TEXT);
 }
 
-void main(void)
+int main(void)
 {
     int mode;
     int i, high;
@@ -165,7 +165,7 @@ void main(void)
 	printf("Choose one of the following video modes: \n");
 
 	high = 0;
-	for (i = 1; i <= GLASTMODE; i++)
+	for (i = 1; i <= vga_lastmodenumber(); i++)
 	    if (vga_hasmode(i)) {
 		vga_modeinfo *info;
 		char expl[100];
