@@ -169,7 +169,7 @@ void __svgalib_driver8p_fillbox(int x, int y, int w, int h, int c)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w > 0x10000)
+	if (vp + w > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -182,6 +182,7 @@ void __svgalib_driver8p_fillbox(int x, int y, int w, int h, int c)
 		vp = (vp + BYTEWIDTH) & 0xffff;
 		continue;
 	    }
+        };
 	__memset(VBUF + vp, c, w);
 	vp += BYTEWIDTH;
     }
@@ -213,7 +214,7 @@ void __svgalib_driver8p_putbox(int x, int y, int w, int h, void *b, int bw)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w > 0x10000)
+	if (vp + w > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -228,6 +229,7 @@ void __svgalib_driver8p_putbox(int x, int y, int w, int h, void *b, int bw)
 		bp += bw;
 		continue;
 	    }
+        };
 	__memcpy(VBUF + vp, bp, w);
 	bp += bw;
 	vp += BYTEWIDTH;
@@ -259,7 +261,7 @@ void __svgalib_driver8p_getbox(int x, int y, int w, int h, void *b, int bw)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w > 0x10000)
+	if (vp + w > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -274,6 +276,7 @@ void __svgalib_driver8p_getbox(int x, int y, int w, int h, void *b, int bw)
 		bp += bw;
 		continue;
 	    }
+        };
 	__memcpy(bp, VBUF + vp, w);
 	bp += bw;
 	vp += BYTEWIDTH;
@@ -469,7 +472,7 @@ void __svgalib_driver16p_fillbox(int x, int y, int w, int h, int c)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w * 2 > 0x10000)
+	if (vp + w * 2 > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -482,6 +485,7 @@ void __svgalib_driver16p_fillbox(int x, int y, int w, int h, int c)
 		vp = (vp + BYTEWIDTH) & 0xffff;
 		continue;
 	    }
+        };
 	__memset2(VBUF + vp, c, w);
 	vp += BYTEWIDTH;
     }
@@ -750,7 +754,7 @@ void __svgalib_driver24p_fillbox(int x, int y, int w, int h, int c)
     vga_setpage(page);
     if (RGBEQUAL(c)) {
 	for (i = 0; i < h; i++) {
-	    if (vp + w * 3 > 0x10000)
+	    if (vp + w * 3 > 0x10000) {
 		if (vp >= 0x10000) {
 		    page++;
 		    vga_setpage(page);
@@ -763,12 +767,13 @@ void __svgalib_driver24p_fillbox(int x, int y, int w, int h, int c)
 		    vp = (vp + BYTEWIDTH) & 0xffff;
 		    continue;
 		}
+            };
 	    __memset(VBUF + vp, c, w * 3);
 	    vp += BYTEWIDTH;
 	}
     } else
 	for (i = 0; i < h; i++) {
-	    if (vp + w * 3 > 0x10000)
+	    if (vp + w * 3 > 0x10000) {
 		if (vp >= 0x10000) {
 		    page++;
 		    vga_setpage(page);
@@ -798,6 +803,7 @@ void __svgalib_driver24p_fillbox(int x, int y, int w, int h, int c)
 		    vp = (vp + BYTEWIDTH) & 0xffff;
 		    continue;
 		}
+            };
 	    __memset3(VBUF + vp, c, w);
 	    vp += BYTEWIDTH;
 	}
@@ -838,7 +844,7 @@ static void driver24_rev_putbox(int x, int y, int w, int h, void *b, int bw)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w > 0x10000)
+	if (vp + w > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -878,6 +884,7 @@ static void driver24_rev_putbox(int x, int y, int w, int h, void *b, int bw)
 		bp += bw;
 		continue;
 	    }
+        };
 	RGB_swapped_memcpy(VBUF + vp, bp, w);
 	bp += bw;
 	vp += BYTEWIDTH;
@@ -942,7 +949,7 @@ static void driver24_rev_getbox(int x, int y, int w, int h, void *b, int bw)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w > 0x10000)
+	if (vp + w > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -982,6 +989,7 @@ static void driver24_rev_getbox(int x, int y, int w, int h, void *b, int bw)
 		bp += bw;
 		continue;
 	    }
+        };
 	RGB_swapped_memcpy(bp, VBUF + vp, w);
 	bp += bw;
 	vp += BYTEWIDTH;
@@ -1174,7 +1182,7 @@ void __svgalib_driver32p_fillbox(int x, int y, int w, int h, int c)
     vp &= 0xffff;
     vga_setpage(page);
     for (i = 0; i < h; i++) {
-	if (vp + w * 4 > 0x10000)
+	if (vp + w * 4 > 0x10000) {
 	    if (vp >= 0x10000) {
 		page++;
 		vga_setpage(page);
@@ -1187,6 +1195,7 @@ void __svgalib_driver32p_fillbox(int x, int y, int w, int h, int c)
 		vp = (vp + BYTEWIDTH) & 0xffff;
 		continue;
 	    }
+        };
 	__memsetlong(VBUF + vp, c, w);
 	vp += BYTEWIDTH;
     }
@@ -1250,27 +1259,27 @@ void __svgalib_driver32_putboxmask(int x, int y, int w, int h, void *b)
 INLINE void __svgalib_driver32_putboxpart(int x, int y, int w, int h, int ow, int oh,
 				void *b, int xo, int yo)
 {
-    __svgalib_driver32_putbox(x, y, w, h, b + yo * ow + xo, ow);
+    __svgalib_driver32_putbox(x, y, w, h, b + yo * ow * 4 + xo * 4 , ow );
     /* inlined */
 }
 
 INLINE void __svgalib_driver32p_putboxpart(int x, int y, int w, int h, int ow, int oh,
 				 void *b, int xo, int yo)
 {
-    __svgalib_driver32p_putbox(x, y, w, h, b + yo * ow + xo, ow);
+    __svgalib_driver32p_putbox(x, y, w, h, b + yo * ow * 4 + xo * 4, ow );
     /* inlined */
 }
 
 INLINE void __svgalib_driver32_getboxpart(int x, int y, int w, int h, int ow, int oh,
 				void *b, int xo, int yo)
 {
-    __svgalib_driver32_getbox(x, y, w, h, b + yo * ow + xo, ow);
+    __svgalib_driver32_getbox(x, y, w, h, b + yo * ow * 4 + xo * 4 , ow );
 }
 
 INLINE void __svgalib_driver32p_getboxpart(int x, int y, int w, int h, int ow, int oh,
 				 void *b, int xo, int yo)
 {
-    __svgalib_driver32p_getbox(x, y, w, h, b + yo * ow + xo, ow);
+    __svgalib_driver32p_getbox(x, y, w, h, b + yo * ow * 4 + xo * 4 , ow );
 }
 
 INLINE void __svgalib_driver32_copybox(int x1, int y1, int w, int h, int x2, int y2)
