@@ -1,10 +1,8 @@
-
 /* Based on functions in linux/string.h */
 
 #include <linux/types.h>	/* for size_t */
 
-
-#ifdef __alpha__
+#if defined(__alpha__) || defined (NO_ASSEMBLY)
 
 #define __memcpy(dst,src,n)			memcpy((dst),(src),(n))
 #define __memcpy_conventional(dst,src,n)	memcpy((dst),(src),(n))
@@ -20,7 +18,9 @@
 static inline void *
  __memcpy_conventional(void *to, const void *from, size_t n)
 {
-    __asm__("cld\n\t"
+  int dummy1;
+  long dummy2, dummy3;
+    __asm__ __volatile__("cld\n\t"
     	    "cmpl $0,%%edx\n\t"
     	    "jle 2f\n\t"
 	    "movl %%edi,%%ecx\n\t"
@@ -39,8 +39,10 @@ static inline void *
 	    "je 2f\n\t"
 	    "movsw\n"
 	    "2:\n"
-  : :	    "d"(n), "D"((long) to), "S"((long) from)
-  :	    "cx", "dx", "di", "si");
+  :         "=d"(dummy1), "=D"(dummy2), "=S"(dummy3)   /* fake output */ 
+  :	    "0"(n), "1"((long) to), "2"((long) from)
+  :	    "cx"/***rjr***, "dx", "di", "si"***/
+  );
     return (to);
 }
 
@@ -48,10 +50,14 @@ static inline void *
 static inline void *
  __memcpyb(void *to, const void *from, size_t n)
 {
-    __asm__("cld\n\t"
+  int dummy1;
+  long dummy2, dummy3;
+    __asm__ __volatile__("cld\n\t"
 	    "rep ; movsb\n\t"
-  : :	    "c"(n), "D"((long) to), "S"((long) from)
-  :	    "cx", "di", "si");
+  :         "=c"(dummy1), "=D"(dummy2), "=S"(dummy3) /* fake output */
+  :	    "0"(n), "1"((long) to), "2"((long) from)
+			 /***rjr***:	    "cx", "di", "si"***/
+  );
     return (to);
 }
 
@@ -69,18 +75,25 @@ static inline void *
 static inline void *
  __memsetlong(void *s, unsigned c, size_t count)
 {
-    __asm__("cld\n\t"
+  long dummy1;
+  int dummy2;
+    __asm__ __volatile__("cld\n\t"
 	    "rep\n\t"
 	    "stosl"
-  : :	    "a"(c), "D"(s), "c"(count)
-  :	    "cx", "di");
+  :         "=D"(dummy1), "=c"(dummy2) /* fake outputs */
+  :	    "a"(c), "0"(s), "1"(count)
+			 /***rjr***:	    "cx", "di"***/
+  );
     return s;
 }
 
 static inline void *
  __memset(void *s, char c, size_t count)
 {
-    __asm__(
+  int dummy1;
+  long dummy2;
+  int dummy3;
+    __asm__ __volatile__(
 	       "cld\n\t"
 	       "cmpl $12,%%edx\n\t"
 	       "jl 1f\n\t"	/* if (count >= 12) */
@@ -106,8 +119,10 @@ static inline void *
 	       "andl $3,%%edx\n"	/* fill last few bytes */
 	       "1:\tmovl %%edx,%%ecx\n\t"	/* <= 12 entry point */
 	       "rep ; stosb\n\t"
-  : :	       "a"(c), "D"(s), "d"(count)
-  :	       "ax", "cx", "dx", "di");
+  :            "=a"(dummy1), "=D"(dummy2), "=d"(dummy3) /* fake outputs */
+  :	       "0"(c), "1"(s), "2"(count)
+  :	       /***rjr***"ax",*/ "cx"/*, "dx", "di"*/
+  );
     return s;
 }
 
@@ -116,7 +131,10 @@ static inline void *
 /* count is in 16-bit pixels */
 /* s is assumed to be 16-bit aligned */
 {
-    __asm__(
+  int dummy1;
+  long dummy2;
+  int dummy3;
+    __asm__ __volatile__(
 	       "cld\n\t"
 	       "cmpl $12,%%edx\n\t"
 	       "jl 1f\n\t"	/* if (count >= 12) */
@@ -143,8 +161,10 @@ static inline void *
 	       "1:\tmovl %%edx,%%ecx\n\t"	/* <= 12 entry point */
 	       "rep ; stosw\n\t"
 	       "3:\n\t"
-  : :	       "a"(c), "D"(s), "d"(count)
-  :	       "ax", "cx", "dx", "di");
+  :            "=a"(dummy1), "=D"(dummy2), "=d"(dummy3) /* fake outputs */
+  :	       "0"(c), "1"(s), "2"(count)
+  :	       /***rjr***"ax",*/ "cx"/*, "dx", "di"*/
+  );
     return s;
 }
 
@@ -152,7 +172,10 @@ static inline void *
  __memset3(void *s, int c, size_t count)
 /* count is in 24-bit pixels (3 bytes per pixel) */
 {
-    __asm__(
+  int dummy1;
+  long dummy2;
+  int dummy3;
+    __asm__ __volatile__(
 	       "cmpl $8,%%edx\n\t"
     /*      "jmp 2f\n\t" *//* debug */
 	       "jl 2f\n\t"
@@ -224,8 +247,10 @@ static inline void *
 	       "jnz 3b\n\t"
 
 	       "4:\n\t"
-  : :	       "a"(c), "D"(s), "d"(count)
-  :	       "ax", "cx", "dx", "si", "di");
+  :            "=a"(dummy1), "=D"(dummy2), "=d"(dummy3) /* fake outputs */
+  :	       "0"(c), "1"(s), "2"(count)
+  :	       /***rjr***"ax",*/ "cx", /*"dx",*/ "si"/*, "di"*/
+  );
     return s;
 }
 
