@@ -199,10 +199,6 @@ static int apm_modeavailable(int mode)
     struct info *info;
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
-
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24))return 0;
   
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
@@ -214,6 +210,11 @@ static int apm_modeavailable(int mode)
 
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
 
+    if((modeinfo->bitsPerPixel==24)&&(apm_chiptype==AT24)) {
+	free(modeinfo);
+        return 0;
+    }
+
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
@@ -629,7 +630,7 @@ static int apm_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 75000;
     cardspecs->maxPixelClock32bpp = 94500;
     cardspecs->flags = CLOCK_PROGRAMMABLE | INTERLACE_DIVIDE_VERT;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->clocks = NULL;
