@@ -96,7 +96,6 @@ static int ark_baseaddress;	/* PCI base address. */
 static CardSpecs *cardspecs;
 static DacMethods *dac_used;
 static int dac_speed = 0;
-static unsigned char *mmio_base;
 
 static int ark_init(int, int, int);
 
@@ -495,8 +494,8 @@ static int ark_test(void)
 
 static void ark_setpage(int page)
 {
-    outw(0x3c4, 0x15 + (page << 8));	/* Aperture Write index. */
-    outw(0x3c4, 0x16 + (page << 8));	/* Aperture Read index. */
+    outw(SEQ_I, 0x15 + (page << 8));	/* Aperture Write index. */
+    outw(SEQ_I, 0x16 + (page << 8));	/* Aperture Read index. */
 }
 
 
@@ -504,11 +503,11 @@ static void ark_setpage(int page)
 
 static void ark_setdisplaystart(int address)
 {
-    outw(0x3d4, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
-    outw(0x3d4, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
+    outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
+    outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     inb(0x3da);			/* set ATC to addressing mode */
-    outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
+    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
     outb(__svgalib_CRT_I, 0x40);
     outb(__svgalib_CRT_D, (address & 0x1C0000) >> 18);	/* sa18-sa20 */
@@ -520,7 +519,7 @@ static void ark_setdisplaystart(int address)
 
 static void ark_setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
     outb(__svgalib_CRT_I, 0x41);
     outb(__svgalib_CRT_D, (inb(__svgalib_CRT_D) & ~0x08) | ((width >> 3) & 0x100));
 }
@@ -789,9 +788,6 @@ static int ark_init(int force, int par1, int par2)
     __svgalib_clear_accelspecs(__svgalib_ark_driverspecs.accelspecs);
     __svgalib_ark_driverspecs.accelspecs->flags = ACCELERATE_ANY_LINEWIDTH;
     /* Map memory-mapped I/O register space. */
-    mmio_base = valloc(4096);
-    mmap(mmio_base, 4096, PROT_WRITE, MAP_FIXED | MAP_SHARED,
-	 __svgalib_mem_fd, 0xB8000);
 
     if (__svgalib_driver_report) {
 	char *bustype;
@@ -811,6 +807,12 @@ static int ark_init(int force, int par1, int par2)
 	printf("\n");
     }
     __svgalib_driverspecs = &__svgalib_ark_driverspecs;
+    __svgalib_mmio_base = 0xb80000;
+    __svgalib_mmio_size = 4096;
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+    if (ark_bus == PCI)__svgalib_linear_mem_base=ark_baseaddress;
+    __svgalib_linear_mem_size=ark_memory*0x400;
 
     return 0;
 }
@@ -884,64 +886,64 @@ static int ark_init(int force, int par1, int par2)
 #define LINEARDESTADDR		0x0001
 
 #define SETBACKGROUNDCOLOR(c) \
-	*(unsigned short *)(mmio_base + BACKGROUNDCOLOR) = c;
+	*(unsigned short *)(MMIO_POINTER + BACKGROUNDCOLOR) = c;
 
 #define SETFOREGROUNDCOLOR(c) \
-	*(unsigned short *)(mmio_base + FOREGROUNDCOLOR) = c;
+	*(unsigned short *)(MMIO_POINTER + FOREGROUNDCOLOR) = c;
 
 #define SETCOLORMIXSELECT(m) \
-	*(unsigned short *)(mmio_base + COLORMIXSELECT) = m;
+	*(unsigned short *)(MMIO_POINTER + COLORMIXSELECT) = m;
 
 #define SETWRITEPLANEMASK(m) \
-	*(unsigned short *)(mmio_base + WRITEPLANEMASK) = m;
+	*(unsigned short *)(MMIO_POINTER + WRITEPLANEMASK) = m;
 
 #define SETSTENCILPITCH(p) \
-	*(unsigned short *)(mmio_base + STENCILPITCH) = p;
+	*(unsigned short *)(MMIO_POINTER + STENCILPITCH) = p;
 
 #define SETSOURCEPITCH(p) \
-	*(unsigned short *)(mmio_base + SOURCEPITCH) = p;
+	*(unsigned short *)(MMIO_POINTER + SOURCEPITCH) = p;
 
 #define SETDESTPITCH(p) \
-	*(unsigned short *)(mmio_base + DESTPITCH) = p;
+	*(unsigned short *)(MMIO_POINTER + DESTPITCH) = p;
 
 #define SETSTENCILADDR(p) \
-	*(unsigned int *)(mmio_base + STENCILADDR) = p;
+	*(unsigned int *)(MMIO_POINTER + STENCILADDR) = p;
 
 #define SETSOURCEADDR(p) \
-	*(unsigned int *)(mmio_base + SOURCEADDR) = p;
+	*(unsigned int *)(MMIO_POINTER + SOURCEADDR) = p;
 
 #define SETSOURCEXY(x, y) \
-	*(unsigned int *)(mmio_base + SOURCEADDR) = (y << 16) + x;
+	*(unsigned int *)(MMIO_POINTER + SOURCEADDR) = (y << 16) + x;
 
 #define SETSOURCEX(x) \
-	*(unsigned short *)(mmio_base + SOURCEX) = x;
+	*(unsigned short *)(MMIO_POINTER + SOURCEX) = x;
 
 #define SETSOURCEY(y) \
-	*(unsigned short *)(mmio_base + SOURCEY) = y;
+	*(unsigned short *)(MMIO_POINTER + SOURCEY) = y;
 
 #define SETDESTADDR(p) \
-	*(unsigned int *)(mmio_base + DESTADDR) = p;
+	*(unsigned int *)(MMIO_POINTER + DESTADDR) = p;
 
 #define SETDESTXY(x, y) \
-	*(unsigned int *)(mmio_base + DESTADDR) = (y << 16) + x;
+	*(unsigned int *)(MMIO_POINTER + DESTADDR) = (y << 16) + x;
 
 #define SETDESTX(x) \
-	*(unsigned short *)(mmio_base + DESTX) = x;
+	*(unsigned short *)(MMIO_POINTER + DESTX) = x;
 
 #define SETDESTY(y) \
-	*(unsigned short *)(mmio_base + DESTY) = y;
+	*(unsigned short *)(MMIO_POINTER + DESTY) = y;
 
 #define SETWIDTH(p) \
-	*(unsigned short *)(mmio_base + WIDTH) = p - 1;
+	*(unsigned short *)(MMIO_POINTER + WIDTH) = p - 1;
 
 #define SETHEIGHT(p) \
-	*(unsigned short *)(mmio_base + HEIGHT) = p - 1;
+	*(unsigned short *)(MMIO_POINTER + HEIGHT) = p - 1;
 
 #define SETBITMAPCONFIG(p) \
-	*(unsigned short *)(mmio_base + BITMAPCONFIG) = p;
+	*(unsigned short *)(MMIO_POINTER + BITMAPCONFIG) = p;
 
 #define SETCOMMAND(p) \
-	*(unsigned short *)(mmio_base + COMMAND) = p;
+	*(unsigned short *)(MMIO_POINTER + COMMAND) = p;
 
 #define COPISBUSY() (inb(0x3CB) & 0x40)
 
