@@ -18,8 +18,8 @@
 
 #include <stdlib.h>
 #include <stdio.h>		/* for printf */
+#include <unistd.h>
 #include <string.h>		/* for memset */
-#include <sys/mman.h>		/* mmap */
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
@@ -37,6 +37,8 @@
 #include "vgaregs.h"
 #include "interface.h"
 #include "accel.h"
+#include "vgapci.h"
+
 
 #define CIRRUSREG_GR(i) (VGA_TOTAL_REGS + i - VGA_GRAPHICS_COUNT)
 #define CIRRUSREG_SR(i) (VGA_TOTAL_REGS + 5 + i - VGA_SEQUENCER_COUNT)
@@ -105,7 +107,7 @@
 #define CHIP_HAS_32BIT_DRAM_BUS() \
 	CHECKCHIPGREATEREQUAL(CLGD5420B)
 #define CHIP_HAS_64BIT_DRAM_BUS() \
-	CHECKCHIP2(CLGD5434, CLGD5436)
+	CHECKCHIP3(CLGD5434, CLGD5436, CLGD7548)
 #define CHIP_HAS_HIDDENDAC() \
 	CHECKCHIPGREATEREQUAL(CLGD5420B)
 #define CHIP_HAS_ACCELERATION() \
@@ -132,34 +134,37 @@ void __svgalib_cirrusaccel_init(AccelSpecs * accelspecs, int bpp, int width_in_p
 static int cirrus_memory;
 static int cirrus_chiptype;
 static int cirrus_chiprev;
+static int cirrus_pci_linear = 0;
 static unsigned char actualMCLK, programmedMCLK;
 static int DRAMbandwidth, DRAMbandwidthLimit;
-static unsigned char *mmio_base;
 
 
 static CardSpecs *cardspecs;
 
-#define NU_FIXED_CLOCKS 19
+#define NU_FIXED_CLOCKS 21
 
 /* 12.588 clock (0x33, 0x3B) replaced with 12.599 (0x2C, 0x33). */
 
 static int cirrus_fixed_clocks[NU_FIXED_CLOCKS] =
 {
-    12599, 25227, 28325, 31500, 36025, 37747, 39992, 41164,
+    12599, 18000, 19600,
+    25227, 28325, 31500, 36025, 37747, 39992, 41164,
     45076, 49867, 64983, 72163, 75000, 80013, 85226, 89998,
     95019, 100226, 108035
 };
 
 static unsigned char fixed_clock_numerator[NU_FIXED_CLOCKS] =
 {
-    0x2C, 0x4A, 0x5B, 0x42, 0x4E, 0x3A, 0x51, 0x45,
+    0x2C, 0x27, 0x28,
+    0x4A, 0x5B, 0x42, 0x4E, 0x3A, 0x51, 0x45,
     0x55, 0x65, 0x76, 0x7E, 0x6E, 0x5F, 0x7D, 0x58,
     0x49, 0x46, 0x53
 };
 
 static unsigned char fixed_clock_denominator[NU_FIXED_CLOCKS] =
 {
-    0x33, 0x2B, 0x2F, 0x1F, 0x3E, 0x17, 0x3A, 0x30,
+    0x33, 0x3e, 0x3a,
+    0x2B, 0x2F, 0x1F, 0x3E, 0x17, 0x3A, 0x30,
     0x36, 0x3A, 0x34, 0x32, 0x2A, 0x22, 0x2A, 0x1C,
     0x16, 0x14, 0x16
 };
@@ -170,15 +175,15 @@ static unsigned char fixed_clock_denominator[NU_FIXED_CLOCKS] =
  */
 
 enum {
-    CLGD5420 = 0, CLGD5420B, CLGD5422, CLGD5422C, CLGD5424, CLGD5426,
+    CLGD5420 = 0, CLGD7548, CLGD5420B, CLGD5422, CLGD5422C, CLGD5424, CLGD5426,
     CLGD5428, CLGD5429, CLGD5430, CLGD5434, CLGD5436
 };
 
 
 int __svgalib_cirrus_inlinearmode(void)
 {
-    outb(0x3c4, 0x07);
-    return (inb(0x3c5) & 0xf0) != 0;
+    outb(SEQ_I, 0x07);
+    return (inb(SEQ_D) & 0xf0) != 0;
 }
 
 /* Fill in chipset specific mode information */
@@ -385,6 +390,10 @@ static int cirrus_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
+    /* CLGD7548 mode 13 is broken. Remove this if when supported */
+    if (mode == G320x200x256 && cirrus_chiptype == CLGD7548)
+ return 0;
+
     if ((mode < G640x480x256 && mode != G320x200x256)
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -540,8 +549,8 @@ static void cirrus_initializemode(unsigned char *moderegs,
 	/* CRT FIFO threshold setting. */
 	unsigned char threshold;
 	threshold = 8;
-	if (cirrus_chiptype >= CLGD5434)
-	    threshold = 1;
+/*	if (cirrus_chiptype >= CLGD5434)
+	    threshold = 1;*/
 	/* XXX Needs more elaborate setting. */
 	SETBITS(moderegs[CIRRUS_PERFTUNING], 0x0F, threshold);
     }
@@ -596,7 +605,7 @@ static int cirrus_setmode(int mode, int prv_mode)
 	/* Let the standard VGA driver set standard VGA modes */
 	/* But first reset an Cirrus extended register that */
 	/* an old XFree86 Trident probe corrupts. */
-	outw(0x3d4, 0x4a0b);
+	outw(CRT_IC, 0x4a0b);
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
     if (!cirrus_modeavailable(mode))
@@ -643,8 +652,8 @@ static void cirrus_unlock(void)
     /* Are we Mono or Color? */
     vgaIOBase = (inb(0x3CC) & 0x01) ? 0x3D0 : 0x3B0;
 
-    outb(0x3C4, 0x06);
-    outb(0x3C5, 0x12);		/* unlock cirrus special */
+    outb(SEQ_I, 0x06);
+    outb(SEQ_D, 0x12);		/* unlock cirrus special */
 
     /* Put the Vert. Retrace End Reg in temp */
 
@@ -665,8 +674,8 @@ static void cirrus_unlock(void)
 
 static void cirrus_lock(void)
 {
-    outb(0x3C4, 0x06);
-    outb(0x3C5, 0x0F);		/* relock cirrus special */
+    outb(SEQ_I, 0x06);
+    outb(SEQ_D, 0x0F);		/* relock cirrus special */
 }
 
 
@@ -677,21 +686,21 @@ static int cirrus_test(void)
     int oldlockreg;
     int lockreg;
 
-    outb(0x3c4, 0x06);
-    oldlockreg = inb(0x3c5);
+    outb(SEQ_I, 0x06);
+    oldlockreg = inb(SEQ_D);
 
     cirrus_unlock();
 
     /* If it's a Cirrus at all, we should be */
     /* able to read back the lock register */
 
-    outb(0x3C4, 0x06);
-    lockreg = inb(0x3C5);
+    outb(SEQ_I, 0x06);
+    lockreg = inb(SEQ_D);
 
     /* Ok, if it's not 0x12, we're not a Cirrus542X. */
     if (lockreg != 0x12) {
-	outb(0x3c4, 0x06);
-	outb(0x3c5, oldlockreg);
+	outb(SEQ_I, 0x06);
+	outb(SEQ_D, oldlockreg);
 	return 0;
     }
     /* The above check seems to be weak, so we also check the chip ID. */
@@ -707,10 +716,12 @@ static int cirrus_test(void)
     case 0x28:			/* 5430 */
     case 0x2A:			/* 5434 */
     case 0x2B:			/* 5436 */
+    case 0x2E:			/* 5446 */
+    case 0x0E:			/* 7548 */
 	break;
     default:
-	outb(0x3c4, 0x06);
-	outb(0x3c5, oldlockreg);
+	outb(SEQ_I, 0x06);
+	outb(SEQ_D, oldlockreg);
 	return 0;
     }
 
@@ -725,13 +736,13 @@ static int cirrus_test(void)
 static void cirrus_setpage_2M(int page)
 {
     /* Cirrus banking register has been set to 16K granularity */
-    outw(0x3ce, (page << 10) + 0x09);
+    outw(GRA_I, (page << 10) + 0x09);
 }
 
 static void cirrus_setpage(int page)
 {
     /* default 4K granularity */
-    outw(0x3ce, (page << 12) + 0x09);
+    outw(GRA_I, (page << 12) + 0x09);
 }
 
 
@@ -749,21 +760,21 @@ static void cirrus_setwrpage(int page)
 
 static void cirrus_setdisplaystart(int address)
 {
-    outw(0x3d4, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
-    outw(0x3d4, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
+    outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
+    outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     inb(0x3da);			/* set ATC to addressing mode */
-    outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
+    outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
     /* Cirrus specific bits 0,1 and 18,19,20: */
-    outb(0x3c0, (inb(0x3c1) & 0xf0) | (address & 3));
+    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 3));
     /* write sa0-1 to bits 0-1; other cards use bits 1-2 */
-    outb(0x3d4, 0x1b);
-    outb(0x3d5, (inb(0x3d5) & 0xf2)
+    outb(CRT_IC, 0x1b);
+    outb(CRT_DC, (inb(CRT_DC) & 0xf2)
 	 | ((address & 0x40000) >> 18)	/* sa18: write to bit 0 */
 	 |((address & 0x80000) >> 17)	/* sa19: write to bit 2 */
 	 |((address & 0x100000) >> 17));	/* sa20: write to bit 3 */
-    outb(0x3d4, 0x1d);
+    outb(CRT_IC, 0x1d);
     if (cirrus_memory > 2048)
-	outb(0x3d5, (inb(0x3d5) & 0x7f)
+	outb(CRT_DC, (inb(CRT_DC) & 0x7f)
 	     | ((address & 0x200000) >> 14));	/* sa21: write to bit 7 */
 }
 
@@ -773,18 +784,18 @@ static void cirrus_setdisplaystart(int address)
 
 static void cirrus_setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
-    outb(0x3d4, 0x1b);
-    outb(0x3d5, (inb(0x3d5) & 0xef) | ((width & 0x800) >> 7));
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outb(CRT_IC, 0x1b);
+    outb(CRT_DC, (inb(CRT_DC) & 0xef) | ((width & 0x800) >> 7));
     /* write lw12 to bit 4 of Sequencer reg. 0x1b */
 }
 
 static void cirrus_setlinear(int addr)
 {
     int val;
-    outb(0x3c4, 0x07);
-    val = inb(0x3c5);
-    outb(0x3c5, (val & 0x0f) | (addr << 4));
+    outb(SEQ_I, 0x07);
+    val = inb(SEQ_D);
+    outb(SEQ_D, (val & 0x0f) | (addr << 4));
 }
 
 static int cirrus_linear(int op, int param)
@@ -823,6 +834,7 @@ static int cirrus_linear(int op, int param)
     }
     if (cirrus_chiptype >= CLGD5430) {
 	if (op == LINEAR_QUERY_BASE) {
+            if(cirrus_pci_linear)return cirrus_pci_linear;
 	    if (param == 0)
 		return 0x04000000;	/* 64MB */
 	    if (param == 1)
@@ -839,7 +851,7 @@ static int cirrus_linear(int op, int param)
 	    if (param == 5)
 		return 0xA8000000;
 	    if (param == 6)
-		return 0xF0000000;
+		return 0xE0000000;
 	    if (param == 7)
 		return 0XFE000000;
 	    /*
@@ -895,8 +907,8 @@ DriverSpecs __svgalib_cirrus_driverspecs =
 
 static char *cirrus_chipname[] =
 {
-    "5420", "5420-75QC-B", "5422", "5422-80QC-C", "5424", "5426", "5428",
-    "5429", "5430", "5434", "5436"
+    "5420", "7548", "5420-75QC-B", "5422", "5422-80QC-C", "5424", "5426", 
+    "5428", "5429", "5430", "5434", "5436"
 };
 
 static unsigned char fixedMCLK[4] =
@@ -971,6 +983,9 @@ static int cirrus_init(int force, int par1, int par2)
 	    /* Treat 5446 as 5436. */
 	    cirrus_chiptype = CLGD5436;
 	    break;
+	case 0x0E:
+	    cirrus_chiptype = CLGD7548;
+	    break;
 	default:
 	    printf("Unknown Cirrus chip %2x.\n",
 		   cirrus_chiptype);
@@ -988,9 +1003,9 @@ static int cirrus_init(int force, int par1, int par2)
 #endif
 
 	/* Now determine the amount of memory. */
-	outb(0x3c4, 0x0a);	/* read memory register */
+	outb(SEQ_I, 0x0a);	/* read memory register */
 	/* This depends on the BIOS having set a scratch register. */
-	v = inb(0x3c5);
+	v = inb(SEQ_D);
 	cirrus_memory = 256 << ((v >> 3) & 3);
 
 	/* Determine memory the correct way for the 543x, and
@@ -999,8 +1014,8 @@ static int cirrus_init(int force, int par1, int par2)
 				       && cirrus_chiptype != CLGD5420)) {
 	    unsigned char SRF;
 	    cirrus_memory = 512;
-	    outb(0x3c4, 0x0f);
-	    SRF = inb(0x3c5);
+	    outb(SEQ_I, 0x0f);
+	    SRF = inb(SEQ_D);
 	    if (SRF & 0x10)
 		/* 32-bit DRAM bus. */
 		cirrus_memory *= 2;
@@ -1045,6 +1060,23 @@ static int cirrus_init(int force, int par1, int par2)
      */
     DRAMbandwidthLimit = (DRAMbandwidth * 10) / 11;
 
+    if(cirrus_chiptype>=CLGD5430) {
+       int i;
+        
+       i=(__svgalib_inSR(0x17)>>3)&7;
+       if(i==4) {
+          unsigned long buf[64];
+          int found, _ioperm=0;
+         
+          if(getenv("IOPERM")==NULL) {
+             _ioperm=1;
+             iopl(3);
+          };
+          found=__svgalib_pci_find_vendor_vga(0x1013,buf,0);
+          if(_ioperm)iopl(0);
+          if(!found)cirrus_pci_linear=buf[4]&0xffff0000;
+       };
+    };
 /* begin: Initialize card specs. */
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = cirrus_memory;
@@ -1150,12 +1182,6 @@ static int cirrus_init(int force, int par1, int par2)
     __svgalib_cirrus_driverspecs.accelspecs = malloc(sizeof(AccelSpecs));
     __svgalib_clear_accelspecs(__svgalib_cirrus_driverspecs.accelspecs);
     __svgalib_cirrus_driverspecs.accelspecs->flags = ACCELERATE_ANY_LINEWIDTH;
-    if (cirrus_chiptype >= CLGD5429) {
-	/* Map memory-mapped I/O register space. */
-	mmio_base = valloc(32 * 1024);
-	mmap(mmio_base, 32 * 1024, PROT_WRITE, MAP_FIXED | MAP_SHARED,
-	     __svgalib_mem_fd, 0xB8000);
-    }
     /* Set up the correct paging routine */
     if (cirrus_memory >= 2048)
 	__svgalib_cirrus_driverspecs.__svgalib_setpage =
@@ -1163,6 +1189,12 @@ static int cirrus_init(int force, int par1, int par2)
 
     __svgalib_driverspecs = &__svgalib_cirrus_driverspecs;
 
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+    __svgalib_linear_mem_base=cirrus_pci_linear;
+    __svgalib_linear_mem_size=cirrus_memory*0x400;
+    __svgalib_mmio_size=32768;
+    __svgalib_mmio_base=0xb8000;
     return 0;
 }
 
@@ -1174,30 +1206,30 @@ static int cirrus_init(int force, int par1, int par2)
    port index
 
    Addresses have 21 bits (2Mb of memory).
-   0x3ce,       0x28    bits 0-7 of the destination address
-   0x3ce,  0x29 bits 8-15
-   0x3ce,       0x2a    bits 16-20
+   GRA_I,       0x28    bits 0-7 of the destination address
+   GRA_I,  0x29 bits 8-15
+   GRA_I,       0x2a    bits 16-20
 
-   0x3ce,       0x2c    bits 0-7 of the source address
-   0x3ce,       0x2d    bits 8-15
-   0x3ce,       0x2e    bits 16-20
+   GRA_I,       0x2c    bits 0-7 of the source address
+   GRA_I,       0x2d    bits 8-15
+   GRA_I,       0x2e    bits 16-20
 
    Maximum pitch is 4095.
-   0x3ce,       0x24    bits 0-7 of the destination pitch (screen width)
-   0x3ce,       0x25    bits 8-11
+   GRA_I,       0x24    bits 0-7 of the destination pitch (screen width)
+   GRA_I,       0x25    bits 8-11
 
-   0x3ce,       0x26    bits 0-7 of the source pitch (screen width)
-   0x3ce,       0x27    bits 8-11
+   GRA_I,       0x26    bits 0-7 of the source pitch (screen width)
+   GRA_I,       0x27    bits 8-11
 
    Maximum width is 2047.
-   0x3ce,       0x20    bits 0-7 of the box width - 1
-   0x3ce,       0x21    bits 8-10
+   GRA_I,       0x20    bits 0-7 of the box width - 1
+   GRA_I,       0x21    bits 8-10
 
    Maximum height is 1023.
-   0x3ce,       0x22    bits 0-7 of the box height - 1
-   0x3ce,       0x23    bits 8-9
+   GRA_I,       0x22    bits 0-7 of the box height - 1
+   GRA_I,       0x23    bits 8-9
 
-   0x3ce,       0x30    BLT mode
+   GRA_I,       0x30    BLT mode
    bit 0: direction (0 = down, 1 = up)
    bit 1: destination
    bit 2: source (0 = video memory, 1 = system memory)
@@ -1225,15 +1257,15 @@ static int cirrus_init(int force, int par1, int par2)
    0x34 BLT transparent color
    0x35 high byte
 
-   0x3ce,  0x00 background color (for color expansion)
-   0x3ce,  0x01 foreground color
-   0x3ce,  0x10 high byte of background color for 16-bit pixels
-   0x3ce,  0x11 high byte of foreground color
+   GRA_I,  0x00 background color (for color expansion)
+   GRA_I,  0x01 foreground color
+   GRA_I,  0x10 high byte of background color for 16-bit pixels
+   GRA_I,  0x11 high byte of foreground color
 
-   0x3ce,       0x0b    bit 1: enable BY8 addressing
-   0x3c4,       0x02    8-bit plane mask for BY8 (corresponds to 8 pixels)
+   GRA_I,       0x0b    bit 1: enable BY8 addressing
+   SEQ_I,       0x02    8-bit plane mask for BY8 (corresponds to 8 pixels)
    (write mode 1, 4, 5)
-   0x3c5,  0x05 bits 0-2: VGA write mode
+   SEQ_D,  0x05 bits 0-2: VGA write mode
    extended write mode 4: write up to 8 pixels in
    foreground color (BY8)
    extended write mode 5: write 8 pixels in foreground/
@@ -1277,94 +1309,94 @@ static int cirrus_init(int force, int par1, int par2)
 /* Macros for normal I/O BitBLT register access. */
 
 #define SETSRCADDR(addr) \
-	outw(0x3CE, (((addr) & 0x000000FF) << 8) | 0x2C); \
-	outw(0x3CE, (((addr) & 0x0000FF00)) | 0x2D); \
-	outw(0x3CE, (((addr) & 0x003F0000) >> 8) | 0x2E);
+	outw(GRA_I, (((addr) & 0x000000FF) << 8) | 0x2C); \
+	outw(GRA_I, (((addr) & 0x0000FF00)) | 0x2D); \
+	outw(GRA_I, (((addr) & 0x003F0000) >> 8) | 0x2E);
 
 #define SETDESTADDR(addr) \
-	outw(0x3CE, (((addr) & 0x000000FF) << 8) | 0x28); \
-	outw(0x3CE, (((addr) & 0x0000FF00)) | 0x29); \
-	outw(0x3CE, (((addr) & 0x003F0000) >> 8) | 0x2A);
+	outw(GRA_I, (((addr) & 0x000000FF) << 8) | 0x28); \
+	outw(GRA_I, (((addr) & 0x0000FF00)) | 0x29); \
+	outw(GRA_I, (((addr) & 0x003F0000) >> 8) | 0x2A);
 
 /* Pitch: the 5426 goes up to 4095, the 5434 can do 8191. */
 
 #define SETDESTPITCH(pitch) \
-	outw(0x3CE, (((pitch) & 0x000000FF) << 8) | 0x24); \
-	outw(0x3CE, (((pitch) & 0x00001F00)) | 0x25);
+	outw(GRA_I, (((pitch) & 0x000000FF) << 8) | 0x24); \
+	outw(GRA_I, (((pitch) & 0x00001F00)) | 0x25);
 
 #define SETSRCPITCH(pitch) \
-	outw(0x3CE, (((pitch) & 0x000000FF) << 8) | 0x26); \
-	outw(0x3CE, (((pitch) & 0x00001F00)) | 0x27);
+	outw(GRA_I, (((pitch) & 0x000000FF) << 8) | 0x26); \
+	outw(GRA_I, (((pitch) & 0x00001F00)) | 0x27);
 
 /* Width: the 5426 goes up to 2048, the 5434 can do 8192. */
 
 #define SETWIDTH(width) \
-	outw(0x3CE, ((((width) - 1) & 0x000000FF) << 8) | 0x20); \
-	outw(0x3CE, ((((width) - 1) & 0x00001F00)) | 0x21);
+	outw(GRA_I, ((((width) - 1) & 0x000000FF) << 8) | 0x20); \
+	outw(GRA_I, ((((width) - 1) & 0x00001F00)) | 0x21);
 
 /* Height: the 5426 goes up to 1024, the 5434 can do 2048. */
 /* It appears many 5434's only go up to 1024. */
 
 #define SETHEIGHT(height) \
-	outw(0x3CE, ((((height) - 1) & 0x000000FF) << 8) | 0x22); \
-	outw(0x3CE, (((height) - 1) & 0x00000700) | 0x23);
+	outw(GRA_I, ((((height) - 1) & 0x000000FF) << 8) | 0x22); \
+	outw(GRA_I, (((height) - 1) & 0x00000700) | 0x23);
 
 #define SETBLTMODE(m) \
-	outw(0x3CE, ((m) << 8) | 0x30);
+	outw(GRA_I, ((m) << 8) | 0x30);
 
 #define SETBLTWRITEMASK(m) \
-	outw(0x3CE, ((m) << 8) | 0x2F);
+	outw(GRA_I, ((m) << 8) | 0x2F);
 
 #define SETTRANSPARENCYCOLOR(c) \
-	outw(0x3CE, ((c) << 8) | 0x34);
+	outw(GRA_I, ((c) << 8) | 0x34);
 
 #define SETTRANSPARENCYCOLOR16(c) \
-	outw(0x3CE, ((c) << 8) | 0x34); \
-	outw(0x3CE, (c & 0xFF00) | 0x35);
+	outw(GRA_I, ((c) << 8) | 0x34); \
+	outw(GRA_I, (c & 0xFF00) | 0x35);
 
 #define SETTRANSPARENCYCOLORMASK16(m) \
-	outw(0x3CE, ((m) << 8) | 0x38); \
-	outw(0x3CE, ((m) & 0xFF00) | 0x39);
+	outw(GRA_I, ((m) << 8) | 0x38); \
+	outw(GRA_I, ((m) & 0xFF00) | 0x39);
 
 #define SETROP(rop) \
-	outw(0x3CE, ((rop) << 8) | 0x32);
+	outw(GRA_I, ((rop) << 8) | 0x32);
 
 #define SETFOREGROUNDCOLOR(c) \
-	outw(0x3CE, 0x01 + ((c) << 8));
+	outw(GRA_I, 0x01 + ((c) << 8));
 
 #define SETBACKGROUNDCOLOR(c) \
-	outw(0x3CE, 0x00 + ((c) << 8));
+	outw(GRA_I, 0x00 + ((c) << 8));
 
 #define SETFOREGROUNDCOLOR16(c) \
-	outw(0x3CE, 0x01 + ((c) << 8)); \
-	outw(0x3CE, 0x11 + ((c) & 0xFF00));
+	outw(GRA_I, 0x01 + ((c) << 8)); \
+	outw(GRA_I, 0x11 + ((c) & 0xFF00));
 
 #define SETBACKGROUNDCOLOR16(c) \
-	outw(0x3CE, 0x00 + ((c) << 8)); \
-	outw(0x3CE, 0x10 + ((c) & 0xFF00)); \
+	outw(GRA_I, 0x00 + ((c) << 8)); \
+	outw(GRA_I, 0x10 + ((c) & 0xFF00)); \
 
 #define SETFOREGROUNDCOLOR32(c) \
-	outw(0x3CE, 0x01 + ((c) << 8)); \
-	outw(0x3CE, 0x11 + ((c) & 0xFF00)); \
-	outw(0x3CE, 0x13 + (((c) & 0xFf0000) >> 8)); \
-	outw(0x3CE, 0x15 + (((unsigned int)(c) & 0xFF000000) >> 16));
+	outw(GRA_I, 0x01 + ((c) << 8)); \
+	outw(GRA_I, 0x11 + ((c) & 0xFF00)); \
+	outw(GRA_I, 0x13 + (((c) & 0xFf0000) >> 8)); \
+	outw(GRA_I, 0x15 + (((unsigned int)(c) & 0xFF000000) >> 16));
 
 #define SETBACKGROUNDCOLOR32(c) \
-	outw(0x3CE, 0x00 + ((c) << 8)); \
-	outw(0x3CE, 0x10 + ((c) & 0xFF00)); \
-	outw(0x3CE, 0x12 + (((c) & 0xFF0000) >> 8)); \
-	outw(0x3CE, 0x14 + (((unsigned int)(c) & 0xFF000000) >> 16));
+	outw(GRA_I, 0x00 + ((c) << 8)); \
+	outw(GRA_I, 0x10 + ((c) & 0xFF00)); \
+	outw(GRA_I, 0x12 + (((c) & 0xFF0000) >> 8)); \
+	outw(GRA_I, 0x14 + (((unsigned int)(c) & 0xFF000000) >> 16));
 
 #define STARTBLT() { \
 	unsigned char tmp; \
-	outb(0x3CE, 0x31); \
-	tmp = inb(0x3CF); \
-	outb(0x3CF, tmp | 0x02); \
+	outb(GRA_I, 0x31); \
+	tmp = inb(GRA_D); \
+	outb(GRA_D, tmp | 0x02); \
 	}
 
 #define BLTBUSY(s) { \
-	outb(0x3CE, 0x31); \
-	s = inb(0x3CF) & 1; \
+	outb(GRA_I, 0x31); \
+	s = inb(GRA_D) & 1; \
 	}
 
 #define WAITUNTILFINISHED() \
@@ -1394,61 +1426,61 @@ static int cirrus_init(int force, int par1, int par2)
 #define MMIOBLTSTATUS		0x40
 
 #define MMIOSETDESTADDR(addr) \
-  *(unsigned int *)(mmio_base + MMIODESTADDR) = addr;
+  *(unsigned int *)(MMIO_POINTER + MMIODESTADDR) = addr;
 
 #define MMIOSETSRCADDR(addr) \
-  *(unsigned int *)(mmio_base + MMIOSRCADDR) = addr;
+  *(unsigned int *)(MMIO_POINTER + MMIOSRCADDR) = addr;
 
 /* Pitch: the 5426 goes up to 4095, the 5434 can do 8191. */
 
 #define MMIOSETDESTPITCH(pitch) \
-  *(unsigned short *)(mmio_base + MMIODESTPITCH) = pitch;
+  *(unsigned short *)(MMIO_POINTER + MMIODESTPITCH) = pitch;
 
 #define MMIOSETSRCPITCH(pitch) \
-  *(unsigned short *)(mmio_base + MMIOSRCPITCH) = pitch;
+  *(unsigned short *)(MMIO_POINTER + MMIOSRCPITCH) = pitch;
 
 /* Width: the 5426 goes up to 2048, the 5434 can do 8192. */
 
 #define MMIOSETWIDTH(width) \
-  *(unsigned short *)(mmio_base + MMIOWIDTH) = (width) - 1;
+  *(unsigned short *)(MMIO_POINTER + MMIOWIDTH) = (width) - 1;
 
 /* Height: the 5426 goes up to 1024, the 5434 can do 2048. */
 
 #define MMIOSETHEIGHT(height) \
-  *(unsigned short *)(mmio_base + MMIOHEIGHT) = (height) - 1;
+  *(unsigned short *)(MMIO_POINTER + MMIOHEIGHT) = (height) - 1;
 
 #define MMIOSETBLTMODE(m) \
-  *(unsigned char *)(mmio_base + MMIOBLTMODE) = m;
+  *(unsigned char *)(MMIO_POINTER + MMIOBLTMODE) = m;
 
 #define MMIOSETBLTWRITEMASK(m) \
-  *(unsigned char *)(mmio_base + MMIOBLTWRITEMASK) = m;
+  *(unsigned char *)(MMIO_POINTER + MMIOBLTWRITEMASK) = m;
 
 #define MMIOSETROP(rop) \
-  *(unsigned char *)(mmio_base + MMIOROP) = rop;
+  *(unsigned char *)(MMIO_POINTER + MMIOROP) = rop;
 
 #define MMIOSTARTBLT() \
-  *(unsigned char *)(mmio_base + MMIOBLTSTATUS) |= 0x02;
+  *(unsigned char *)(MMIO_POINTER + MMIOBLTSTATUS) |= 0x02;
 
 #define MMIOBLTBUSY(s) \
-  s = *(volatile unsigned char *)(mmio_base + MMIOBLTSTATUS) & 1;
+  s = *(volatile unsigned char *)(MMIO_POINTER + MMIOBLTSTATUS) & 1;
 
 #define MMIOSETBACKGROUNDCOLOR(c) \
-  *(unsigned char *)(mmio_base + MMIOBACKGROUNDCOLOR) = c;
+  *(unsigned char *)(MMIO_POINTER + MMIOBACKGROUNDCOLOR) = c;
 
 #define MMIOSETFOREGROUNDCOLOR(c) \
-  *(unsigned char *)(mmio_base + MMIOFOREGROUNDCOLOR) = c;
+  *(unsigned char *)(MMIO_POINTER + MMIOFOREGROUNDCOLOR) = c;
 
 #define MMIOSETBACKGROUNDCOLOR16(c) \
-  *(unsigned short *)(mmio_base + MMIOBACKGROUNDCOLOR) = c;
+  *(unsigned short *)(MMIO_POINTER + MMIOBACKGROUNDCOLOR) = c;
 
 #define MMIOSETFOREGROUNDCOLOR16(c) \
-  *(unsigned short *)(mmio_base + MMIOFOREGROUNDCOLOR) = c;
+  *(unsigned short *)(MMIO_POINTER + MMIOFOREGROUNDCOLOR) = c;
 
 #define MMIOSETBACKGROUNDCOLOR32(c) \
-  *(unsigned int *)(mmio_base + MMIOBACKGROUNDCOLOR) = c;
+  *(unsigned int *)(MMIO_POINTER + MMIOBACKGROUNDCOLOR) = c;
 
 #define MMIOSETFOREGROUNDCOLOR32(c) \
-  *(unsigned int *)(mmio_base + MMIOFOREGROUNDCOLOR) = c;
+  *(unsigned int *)(MMIO_POINTER + MMIOFOREGROUNDCOLOR) = c;
 
 #define MMIOWAITUNTILFINISHED() \
 	for (;;) { \
