@@ -29,4 +29,4 @@ typedef struct {
     int DAC_stateSize;
 } ClockChipMethods;
 
-extern ClockChipMethods I2061A_clockchip_methods;
+extern ClockChipMethods __svgalib_I2061A_clockchip_methods;
