@@ -17,8 +17,6 @@
 #include "driver.h"
 #include "vga.h"
 
-#include <asm/system.h>
-
 /* 
  * ATTENTION: The ICD 2061A does not support reading of the currently selected
  * pixelclock. XFree86 also fails to restore this value correctly, but always
@@ -155,10 +153,6 @@ static void I2061A_init_clock(unsigned long setup)
 
     outw(SEQ_I, 0x0100);
 
-    cli();
-
-    cli();
-
     wrt_clk_bit(oldclk | 0x08);
     wrt_clk_bit(oldclk | 0x0C);
     for (i = 0; i < 5; i++) {
@@ -182,10 +176,6 @@ static void I2061A_init_clock(unsigned long setup)
     wrt_clk_bit(nclk[1]);
     wrt_clk_bit(clk[1]);
 
-    sti();
-
-    sti();
-
     outb(SEQ_I, 1);
     c = inb(SEQ_D);
     outb(SEQ_D, 0xDF & c);
