@@ -88,6 +88,11 @@ extern DriverSpecs __svgalib_rage_driverspecs;
 extern DriverSpecs __svgalib_banshee_driverspecs;
 extern DriverSpecs __svgalib_sis_driverspecs;
 extern DriverSpecs __svgalib_i740_driverspecs;
+extern DriverSpecs __svgalib_laguna_driverspecs;
+extern DriverSpecs __svgalib_fbdev_driverspecs;
+extern DriverSpecs __svgalib_g400_driverspecs;
+extern DriverSpecs __svgalib_r128_driverspecs;
+extern DriverSpecs __svgalib_savage_driverspecs;
 
 extern DriverSpecs *__svgalib_driverspecs;
 extern DriverSpecs *__svgalib_driverspecslist[];
