@@ -790,7 +790,7 @@ static void et4000_setdisplaystart(int address)
     outb(CRT_IC, 0x33);
     if (et4000_chiptype >= CHIP_ET4000W32i)
 	/* write sa18-21 to bit 0-3 */
-	outb(CRT_DC, (inb(CRT_DC) & 0xf0) | ((address & ATT_IW000) >> 18));
+	outb(CRT_DC, (inb(CRT_DC) & 0xf0) | ((address & 0x3c0000) >> 18));
     else
 	/* write sa18-19 to bit 0-3 */
 	outb(CRT_DC, (inb(CRT_DC) & 0xfc) | ((address & 0xc0000) >> 18));
