@@ -7,8 +7,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/types.h>
-#include <sys/mman.h>
-#include <asm/page.h>
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
