@@ -114,8 +114,8 @@ static void gvga6400_setpage(int page)
     page &= 7;
     last_page = 0x40 | page | (page << 3);
 
-    outb(0x3c4, 6);
-    outb(0x3c5, last_page);
+    outb(SEQ_I, 6);
+    outb(SEQ_D, last_page);
 }
 
 static void gvga6400_setrdpage(int page)
@@ -123,8 +123,8 @@ static void gvga6400_setrdpage(int page)
     last_page &= 0xf8;
     last_page |= page & 0x07;
 
-    outb(0x3c4, 6);
-    outb(0x3c5, last_page);
+    outb(SEQ_I, 6);
+    outb(SEQ_D, last_page);
 }
 
 static void gvga6400_setwrpage(int page)
@@ -132,8 +132,8 @@ static void gvga6400_setwrpage(int page)
     last_page &= 0xc7;
     last_page |= ((page & 7) << 3);
 
-    outb(0x3c4, 6);
-    outb(0x3c5, last_page);
+    outb(SEQ_I, 6);
+    outb(SEQ_D, last_page);
 }
 
 
@@ -148,26 +148,26 @@ static void gvga6400_setdisplaystart(int address)
 	switch (modeinfo.colors) {
 	case 16:		/* planar 16-color mode */
 	    inb(0x3da);
-	    outb(0x3c0, 0x13 + 0x20);
-	    outb(0x3c0, (inb(0x3c1) & 0xf0) | (address & 7));
+	    outb(ATT_IW, 0x13 + 0x20);
+	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 7));
 	    /* write sa0-2 to bits 0-2 */
 	    address >>= 3;
 	    break;
 	case 256:		/* planar 256-color mode */
 	    inb(0x3da);
-	    outb(0x3c0, 0x13 + 0x20);
-	    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+	    outb(ATT_IW, 0x13 + 0x20);
+	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
 	    /* write sa0-1 to bits 1-2 */
 	    address >>= 2;
 	    break;
 	}
-    outw(0x3d4, 0x0d + (address & 0x00ff) * 256);	/* sa0-sa7 */
-    outw(0x3d4, 0x0c + (address & 0xff00));	/* sa8-sa15 */
+    outw(CRT_IC, 0x0d + (address & 0x00ff) * 256);	/* sa0-sa7 */
+    outw(CRT_IC, 0x0c + (address & 0xff00));	/* sa8-sa15 */
 }
 
 static void gvga6400_setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
 }
 
 
@@ -218,36 +218,36 @@ static int gvga6400_test(void)
 
 static int gvga6400_saveregs(unsigned char regs[])
 {
-    outb(0x3c4, 6);
-    regs[EXT + 2] = inb(0x3c5);
-    outb(0x3c4, 7);
-    regs[EXT + 3] = inb(0x3c5);
-    outb(0x3c4, 8);
-    regs[EXT + 4] = inb(0x3c5);
-    outb(0x3c4, 0x10);
-    regs[EXT + 5] = inb(0x3c5);
-    outb(0x3ce, 9);
-    regs[EXT + 6] = inb(0x3cf);
-    outb(0x3d4, 0x2f);
-    regs[EXT + 10] = inb(0x3d5);
+    outb(SEQ_I, 6);
+    regs[EXT + 2] = inb(SEQ_D);
+    outb(SEQ_I, 7);
+    regs[EXT + 3] = inb(SEQ_D);
+    outb(SEQ_I, 8);
+    regs[EXT + 4] = inb(SEQ_D);
+    outb(SEQ_I, 0x10);
+    regs[EXT + 5] = inb(SEQ_D);
+    outb(GRA_I, 9);
+    regs[EXT + 6] = inb(GRA_D);
+    outb(CRT_IC, 0x2f);
+    regs[EXT + 10] = inb(CRT_DC);
 
     return (11);
 }
 
 static void gvga6400_setregs(const unsigned char regs[], int mode)
 {
-    outb(0x3c4, 6);
-    outb(0x3c5, regs[EXT + 2]);
-    outb(0x3c4, 7);
-    outb(0x3c5, regs[EXT + 3]);
-    outb(0x3c4, 8);
-    outb(0x3c5, regs[EXT + 4]);
-    outb(0x3c4, 0x10);
-    outb(0x3c5, regs[EXT + 5]);
-    outb(0x3ce, 9);
-    outb(0x3cf, regs[EXT + 6]);
-    outb(0x3d4, 0x2f);
-    outb(0x3d5, regs[EXT + 10]);
+    outb(SEQ_I, 6);
+    outb(SEQ_D, regs[EXT + 2]);
+    outb(SEQ_I, 7);
+    outb(SEQ_D, regs[EXT + 3]);
+    outb(SEQ_I, 8);
+    outb(SEQ_D, regs[EXT + 4]);
+    outb(SEQ_I, 0x10);
+    outb(SEQ_D, regs[EXT + 5]);
+    outb(GRA_I, 9);
+    outb(GRA_D, regs[EXT + 6]);
+    outb(CRT_IC, 0x2f);
+    outb(CRT_DC, regs[EXT + 10]);
 }
 
 DriverSpecs __svgalib_gvga6400_driverspecs =
@@ -285,6 +285,8 @@ static int gvga6400_init(int force, int par1, int par2)
     if (__svgalib_driver_report)
 	printf("Using Genoa GVGA 6400 driver.\n");
     __svgalib_driverspecs = &__svgalib_gvga6400_driverspecs;
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
 
     return 0;
 }
