@@ -247,8 +247,6 @@ static int i740_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -729,7 +727,7 @@ static int i740_init(int force, int par1, int par2)
     };
 
     if (__svgalib_driver_report) {
-	printf("Using I740 driver, %iKB. ",i740_memory);
+	printf("Using I740 driver, %iKB.\n",i740_memory);
     };
 
     cardspecs = malloc(sizeof(CardSpecs));
@@ -740,7 +738,7 @@ static int i740_init(int force, int par1, int par2)
     cardspecs->maxPixelClock24bpp = 128000;
     cardspecs->maxPixelClock32bpp = 86000;
     cardspecs->flags = CLOCK_PROGRAMMABLE;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->mapClock = i740_map_clock;
