@@ -7,17 +7,17 @@
  * structure based on a svgalib mode number.
  */
 
-ModeInfo *createModeInfoStructureForSvgalibMode(int mode);
+ModeInfo *__svgalib_createModeInfoStructureForSvgalibMode(int mode);
 
 /*
  * This function converts a number of significant color bits to a matching
  * DAC mode type as defined in the RAMDAC interface.
  */
 
-int colorbits_to_colormode(int bpp, int colorbits);
+int __svgalib_colorbits_to_colormode(int bpp, int colorbits);
 
 /*
  * Clear the accelspecs structure (disable acceleration).
  */
 
-void clear_accelspecs(AccelSpecs * accelspecs);
+void __svgalib_clear_accelspecs(AccelSpecs * accelspecs);
