@@ -175,7 +175,7 @@ static int joystick_upd1(int joydev)
    retval = 2; /* further events might be pending */
 
    if ((js.type & JS_EVENT_AXIS) == JS_EVENT_AXIS) {
-     joydesc[joydev].joydata->handler(JOY_EVENTAXIS, js.number, (js.value-0x8000)>>8, joydev);
+     joydesc[joydev].joydata->handler(JOY_EVENTAXIS, js.number, js.value>>8, joydev);
      retval = 3;
    }
 
@@ -571,6 +571,7 @@ int joystick_init(int joydev, __joystick_output jo)
 	     joystick_initdefhandler(joydev, buttons, axes);
 	     
 	     /* Now calibrate ... */
+#if 0 /* assume joystick is calibrated */
 	     if (jo)
 		jscal1(joydev, jo);
 
@@ -581,6 +582,7 @@ int joystick_init(int joydev, __joystick_output jo)
 	       joystick_close(joydev);
 	       return -1;
 	     }
+#endif
 	     joydesc[joydev].joydata->update = joystick_upd1;
 	     joydesc[joydev].joydata->flip_vc = joystick_flp1;
 	     return 1;
