@@ -12,36 +12,26 @@ This software has NO WARRANTY.  Use it at your own risk.
 
 #include <stdio.h>
 #include <string.h>
-#ifndef SYS_vm86old
-#include <asm/vm86.h> /* required at least for me */
-#else
+#include <asm/vm86.h>
+
+#ifdef USE_LIBC_VM86
 #include <sys/vm86.h>
 #endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <unistd.h>
 #include <fcntl.h>
 
-#include <syscall.h>
-
-#ifndef SYS_vm86old
-#define SYS___svgalib_vm86old SYS_vm86
-#else
-#define SYS___svgalib_vm86old SYS_vm86old
-#endif
-
-
-_syscall1(int, __svgalib_vm86old, struct vm86_struct *, vs)
-
-#define vm86 __svgalib_vm86old
-
 #include "lrmi.h"
 
 #define REAL_MEM_BASE 	((void *)0x10000)
 #define REAL_MEM_SIZE 	0x10000
 #define REAL_MEM_BLOCKS 	0x100
 
+int __svgalib_lrmi_cpu_type=CPU_386;
+
 struct mem_block
 	{
 	unsigned int size : 20;
@@ -61,6 +51,9 @@ real_mem_init(void)
 	void *m;
 	int fd_zero;
 
+	if (mem_info.ready)
+		return 1;
+
 	fd_zero = open("/dev/zero", O_RDONLY);
 	if (fd_zero == -1)
 		{
@@ -111,7 +104,7 @@ delete_block(int i)
 	}
 
 void *
-__svgalib_LRMI_alloc_real(int size)
+LRMI_alloc_real(int size)
 	{
 	int i;
 	char *r = (char *)REAL_MEM_BASE;
@@ -145,7 +138,7 @@ __svgalib_LRMI_alloc_real(int size)
 
 
 void
-__svgalib_LRMI_free_real(void *m)
+LRMI_free_real(void *m)
 	{
 	int i;
 	char *r = (char *)REAL_MEM_BASE;
@@ -191,47 +184,6 @@ static struct
 	} context = { 0 };
 
 
-/*
- These point to the return to 32 bit routine,
- which is defined in the next asm.
-*/
-extern char begin[], end[];
-
-/*
- This function is not intended to be called.
- GCC doesn't allow out of function asms to
- have parameters, so this function is a
- necessary work around.
-*/
-#ifdef SVGA_AOUT
-void
-__svgalib_LRMI__inline_asm(void)
-	{
-	asm volatile (
-	 ".code16\n"
-	 "_begin:\n\t"
-	 "int %0\n\t"
-	 "jmp _begin\n"
-	 "_end:\n"
-	 ".code32"
-	 : : "i" (RETURN_TO_32_INT));
-	}
-#else
-void
-__svgalib_LRMI__inline_asm(void)
-	{
-	asm volatile (
-	 ".code16\n"
-	 "begin:\n\t"
-	 "int %0\n\t"
-	 "jmp begin\n"
-	 "end:\n"
-	 ".code32"
-	 : : "i" (RETURN_TO_32_INT));
-	}
-#endif
-
-
 static inline void
 set_bit(unsigned int bit, void *array)
 	{
@@ -265,16 +217,19 @@ pushw(unsigned short i)
 
 
 int
-__svgalib_LRMI_init(void)
+LRMI_init(void)
 	{
 	void *m;
 	int fd_mem;
 
+	if (context.ready)
+		return 1;
+
 	if (!real_mem_init())
 		return 0;
 
 	/*
-	 Map the Interrupt Vectors (0x0 - 0x400)
+	 Map the Interrupt Vectors (0x0 - 0x400) + BIOS data (0x400 - 0x502)
 	 and the ROM (0xa0000 - 0x100000)
 	*/
 	fd_mem = open("/dev/mem", O_RDWR);
@@ -285,7 +240,7 @@ __svgalib_LRMI_init(void)
 		return 0;
 		}
 
-	m = mmap((void *)0, 0x400,
+	m = mmap((void *)0, 0x502,
 	 PROT_READ | PROT_WRITE | PROT_EXEC,
 	 MAP_FIXED | MAP_PRIVATE, fd_mem, 0);
 
@@ -309,31 +264,34 @@ __svgalib_LRMI_init(void)
 	/*
 	 Allocate a stack
 	*/
-	m = __svgalib_LRMI_alloc_real(DEFAULT_STACK_SIZE);
+	m = LRMI_alloc_real(DEFAULT_STACK_SIZE);
 
 	context.stack_seg = (unsigned int)m >> 4;
 	context.stack_off = DEFAULT_STACK_SIZE;
 
-
 	/*
 	 Allocate the return to 32 bit routine
 	*/
-	m = __svgalib_LRMI_alloc_real(end - begin);
+	m = LRMI_alloc_real(2);
 
 	context.ret_seg = (unsigned int)m >> 4;
 	context.ret_off = (unsigned int)m & 0xf;
 
-	memcpy(m, begin, end - begin);
-
+	((unsigned char *)m)[0] = 0xcd; 	/* int opcode */
+	((unsigned char *)m)[1] = RETURN_TO_32_INT;
 
 	memset(&context.vm, 0, sizeof(context.vm));
 
+	context.vm.cpu_type = __svgalib_lrmi_cpu_type;
+
 	/*
-	 Disable everything but the video int
+	 Enable kernel emulation of all ints except RETURN_TO_32_INT
 	*/
 	memset(&context.vm.int_revectored, 0, sizeof(context.vm.int_revectored));
 	set_bit(RETURN_TO_32_INT, &context.vm.int_revectored);
 
+	context.ready = 1;
+
 	return 1;
 	}
 
@@ -560,7 +518,7 @@ em_inb(void)
 	{
 	asm volatile ("inb (%w1), %b0"
 	 : "=a" (context.vm.regs.eax)
-	 : "d" (context.vm.regs.edx));
+	 : "d" (context.vm.regs.edx), "0" (context.vm.regs.eax));
 	}
 
 static void
@@ -568,7 +526,7 @@ em_inw(void)
 	{
 	asm volatile ("inw (%w1), %w0"
 	 : "=a" (context.vm.regs.eax)
-	 : "d" (context.vm.regs.edx));
+	 : "d" (context.vm.regs.edx), "0" (context.vm.regs.eax));
 	}
 
 static void
@@ -734,6 +692,77 @@ emulate(void)
 	}
 
 
+/*
+ I don't know how to make sure I get the right vm86() from libc.
+ The one I want is syscall # 113 (vm86old() in libc 5, vm86() in glibc)
+ which should be declared as "int vm86(struct vm86_struct *);" in
+ <sys/vm86.h>.
+
+ This just does syscall 113 with inline asm, which should work
+ for both libc's (I hope).
+*/
+#if !defined(USE_LIBC_VM86)
+static int
+lrmi_vm86(struct vm86_struct *vm)
+	{
+	int r;
+#ifdef __PIC__
+	asm volatile (
+	 "pushl %%ebx\n\t"
+	 "movl %2, %%ebx\n\t"
+	 "int $0x80\n\t"
+	 "popl %%ebx"
+	 : "=a" (r)
+	 : "0" (113), "r" (vm));
+#else
+	asm volatile (
+	 "int $0x80"
+	 : "=a" (r)
+	 : "0" (113), "b" (vm));
+#endif
+	return r;
+	}
+#else
+#define lrmi_vm86 vm86
+#endif
+
+
+static void
+debug_info(int vret)
+	{
+	int i;
+	unsigned char *p;
+
+	fputs("vm86() failed\n", stderr);
+	fprintf(stderr, "return = 0x%x\n", vret);
+	fprintf(stderr, "eax = 0x%08lx\n", context.vm.regs.eax);
+	fprintf(stderr, "ebx = 0x%08lx\n", context.vm.regs.ebx);
+	fprintf(stderr, "ecx = 0x%08lx\n", context.vm.regs.ecx);
+	fprintf(stderr, "edx = 0x%08lx\n", context.vm.regs.edx);
+	fprintf(stderr, "esi = 0x%08lx\n", context.vm.regs.esi);
+	fprintf(stderr, "edi = 0x%08lx\n", context.vm.regs.edi);
+	fprintf(stderr, "ebp = 0x%08lx\n", context.vm.regs.ebp);
+	fprintf(stderr, "eip = 0x%08lx\n", context.vm.regs.eip);
+	fprintf(stderr, "cs  = 0x%04x\n", context.vm.regs.cs);
+	fprintf(stderr, "esp = 0x%08lx\n", context.vm.regs.esp);
+	fprintf(stderr, "ss  = 0x%04x\n", context.vm.regs.ss);
+	fprintf(stderr, "ds  = 0x%04x\n", context.vm.regs.ds);
+	fprintf(stderr, "es  = 0x%04x\n", context.vm.regs.es);
+	fprintf(stderr, "fs  = 0x%04x\n", context.vm.regs.fs);
+	fprintf(stderr, "gs  = 0x%04x\n", context.vm.regs.gs);
+	fprintf(stderr, "eflags  = 0x%08lx\n", context.vm.regs.eflags);
+
+	fputs("cs:ip = [ ", stderr);
+
+	p = (unsigned char *)((context.vm.regs.cs << 4) + (context.vm.regs.eip & 0xffff));
+
+	for (i = 0; i < 16; ++i)
+		fprintf(stderr, "%02x ", (unsigned int)p[i]);
+
+	fputs("]\n", stderr);
+	}
+
+
 static int
 run_vm86(void)
 	{
@@ -741,23 +770,41 @@ run_vm86(void)
 
 	while (1)
 		{
-		vret = vm86(&context.vm);
+		vret = lrmi_vm86(&context.vm);
 
-		if (VM86_TYPE(vret) == VM86_INTx
-		 && VM86_ARG(vret) == RETURN_TO_32_INT)
-			return 1;
+		if (VM86_TYPE(vret) == VM86_INTx)
+			{
+			unsigned int v = VM86_ARG(vret);
+
+			if (v == RETURN_TO_32_INT)
+				return 1;
+
+			pushw(context.vm.regs.eflags);
+			pushw(context.vm.regs.cs);
+			pushw(context.vm.regs.eip);
+
+			context.vm.regs.cs = get_int_seg(v);
+			context.vm.regs.eip = get_int_off(v);
+			context.vm.regs.eflags &= ~(VIF_MASK | TF_MASK);
+
+			continue;
+			}
 
 		if (VM86_TYPE(vret) != VM86_UNKNOWN)
-			return 0;
+			break;
 
 		if (!emulate())
-			return 0;
+			break;
 		}
+
+	debug_info(vret);
+
+	return 0;
 	}
 
 
 int
-__svgalib_LRMI_call(struct LRMI_regs *r)
+LRMI_call(struct LRMI_regs *r)
 	{
 	unsigned int vret;
 
@@ -791,7 +838,7 @@ __svgalib_LRMI_call(struct LRMI_regs *r)
 
 
 int
-__svgalib_LRMI_int(int i, struct LRMI_regs *r)
+LRMI_int(int i, struct LRMI_regs *r)
 	{
 	unsigned int vret;
 	unsigned int seg, off;
@@ -803,7 +850,7 @@ __svgalib_LRMI_int(int i, struct LRMI_regs *r)
 	 If the interrupt is in regular memory, it's probably
 	 still pointing at a dos TSR (which is now gone).
 	*/
-	if (seg < 0xa000)
+	if (seg < 0xa000 || (seg << 4) + off >= 0x100000)
 		{
 		fprintf(stderr, "Int 0x%x is not in rom (%04x:%04x)\n", i, seg, off);
 		return 0;
