@@ -8,8 +8,6 @@ do not remove this copyright notice and clearly label modified
 versions as being modified.
 
 This software has NO WARRANTY.  Use it at your own risk.
-
-M. Weller: prepended __svgalib_ to keep these local.
 */
 
 #ifndef LRMI_H
@@ -36,38 +34,52 @@ struct LRMI_regs
 	unsigned short int ss;
 	};
 
+
+#ifndef LRMI_PREFIX
+#define LRMI_PREFIX __svgalib_LRMI_
+#endif
+
+#define LRMI_CONCAT2(a, b) 	a ## b
+#define LRMI_CONCAT(a, b) 	LRMI_CONCAT2(a, b)
+#define LRMI_MAKENAME(a) 	LRMI_CONCAT(LRMI_PREFIX, a)
+
 /*
  Initialize
  returns 1 if sucessful, 0 for failure
 */
+#define LRMI_init LRMI_MAKENAME(init)
 int
-__svgalib_LRMI_init(void);
+LRMI_init(void);
 
 /*
  Simulate a 16 bit far call
  returns 1 if sucessful, 0 for failure
 */
+#define LRMI_call LRMI_MAKENAME(call)
 int
-__svgalib_LRMI_call(struct LRMI_regs *r);
+LRMI_call(struct LRMI_regs *r);
 
 /*
  Simulate a 16 bit interrupt
  returns 1 if sucessful, 0 for failure
 */
+#define LRMI_int LRMI_MAKENAME(int)
 int
-__svgalib_LRMI_int(int interrupt, struct LRMI_regs *r);
+LRMI_int(int interrupt, struct LRMI_regs *r);
 
 /*
  Allocate real mode memory
  The returned block is paragraph (16 byte) aligned
 */
+#define LRMI_alloc_real LRMI_MAKENAME(alloc_real)
 void *
-__svgalib_LRMI_alloc_real(int size);
+LRMI_alloc_real(int size);
 
 /*
  Free real mode memory
 */
+#define LRMI_free_real LRMI_MAKENAME(free_real)
 void
-__svgalib_LRMI_free_real(void *m);
+LRMI_free_real(void *m);
 
 #endif
