@@ -8,8 +8,12 @@
 #include "vgamouse.h"
 #include "../libvga.h"
 
+extern int mouse_open;
+
 void (*__svgalib_mouse_eventhandler) (int, int, int, int, int, int, int) = NULL;
 
+static struct MouseCaps mouse_caps = {0, 0, 0, 0, 0, 0};
+
 #include "ms.c"			/* include low-level mouse driver */
 
 
@@ -30,36 +34,35 @@ int mouse_init_return_fd(char *dev, int type, int samplerate)
 {
     struct sigaction siga;
 
-    if (strcmp(dev, "") == 0)
-	m_dev = "/dev/mouse";
-    else
-	m_dev = dev;
-    m_type = type & MOUSE_TYPE_MASK;
-    m_modem_ctl = type & ~MOUSE_TYPE_MASK;
-
-    m_sample = samplerate;
-
-    currentinthandler = NULL;
-
-    /* Initialize mouse device. */
-    if (m_type == MOUSE_NONE)
-	return -1;
-
-    if (m_type < MOUSE_MICROSOFT || m_type > MOUSE_IMPS2)
-	return -1;
-    if (ms_init())
-	return -1;
-
-    /* Install default mouse handler. Old coordinates are preserved. */
-    __svgalib_mouse_eventhandler = default_handler;
-
-    /* Install interrupt handler. */
-    currentinthandler = inthandler;
-    siga.sa_handler = inthandler;
-    siga.sa_flags = 0;
-    zero_sa_mask(&(siga.sa_mask));
-    sigaction(SIGINT, &siga, &oldsiga);
+    if (!mouse_open) {
+	    if (strcmp(dev, "") == 0)
+		m_dev = "/dev/mouse";
+	    else
+		m_dev = dev;
+	    m_type = type & MOUSE_TYPE_MASK;
+	    m_modem_ctl = type & ~MOUSE_TYPE_MASK;
+	
+	    m_sample = samplerate;
+	
+	    currentinthandler = NULL;
+	
+	    /* Initialize mouse device. */
+	    if (m_type == MOUSE_NONE || m_type < MOUSE_MICROSOFT || m_type > MOUSE_LAST)
+		return -1;
+	    if (ms_init())
+		return -1;
 
+	    /* Install default mouse handler. Old coordinates are preserved. */
+	    __svgalib_mouse_eventhandler = default_handler;
+	
+	    /* Install interrupt handler. */
+	    currentinthandler = inthandler;
+	    siga.sa_handler = inthandler;
+	    siga.sa_flags = 0;
+	    zero_sa_mask(&(siga.sa_mask));
+	    sigaction(SIGINT, &siga, &oldsiga);
+	    mouse_open = 1;
+    }
     return __svgalib_mouse_fd;	/* Return mouse fd. */
 }
 
@@ -68,10 +71,14 @@ int mouse_init_return_fd(char *dev, int type, int samplerate)
 
 int mouse_init(char *dev, int type, int samplerate)
 {
+    if (mouse_open)
+        return 0;
     if (mouse_init_return_fd(dev, type, samplerate) == -1)
 	return -1;
-    else
+    else {
+	mouse_open = 1;
 	return 0;
+    }
 }
 
 /* Reads an event from the mouse buffer.
@@ -108,10 +115,13 @@ void mouse_seteventhandler(__mouse_handler handler)
 
 void mouse_close()
 {
-    ms_close();
-    if (currentinthandler != NULL)
-	/* Restore old interrupt. */
-	sigaction(SIGINT, &oldsiga, NULL);
+    if(mouse_open) {
+        ms_close();
+        if (currentinthandler != NULL)
+            /* Restore old interrupt. */
+	    sigaction(SIGINT, &oldsiga, NULL);
+    }
+    mouse_open=0;
 }
 
 
@@ -143,6 +153,7 @@ static int maxry = 32767;
 static int minrz = 0;
 static int maxrz = 32767;
 static int wrap = 0;
+static int wantcaps = 0;
 
 /* Sets mouse wrap state*/
 void mouse_setwrap(int state)
@@ -272,6 +283,8 @@ void mouse_setposition_6d(int x, int y, int z, int rx, int ry, int rz,
     mouse_ry = ry * scale;
   if (dim_mask & MOUSE_RZDIM)
     mouse_rz = rz * scale;
+  if ((dim_mask & MOUSE_CAPS) && (x == MOUSE_WANTCAPS))
+    wantcaps = 1;
 }
 
 /* Set the mouse range along the x axis. */
@@ -354,6 +367,17 @@ int mouse_gety()
 
 void mouse_getposition_6d(int *x, int *y, int *z, int *rx, int *ry, int *rz)
 {
+    if (wantcaps) {
+        /* Return the mouse capabilities */
+        *x = mouse_caps.key;
+        *y = mouse_caps.buttons;
+        *z = mouse_caps.axes;
+        *rx = mouse_caps.info;
+        *ry = mouse_caps.reserved0;
+        *rz = mouse_caps.reserved1;
+        wantcaps = 0;
+        return;
+    }
   if (x)
     *x = mouse_x / scale;
   if (y)
