@@ -82,6 +82,8 @@ static const unsigned short cflag[] = {
     0,							/* Dummy entry for MOUSE_NONE */
     (CS7 | CREAD | CLOCAL | HUPCL),			/* IntelliMouse (Serial) */
     CS7,       						/* IntelliMouse (PS/2) */
+    (CS7 | CREAD | CLOCAL | HUPCL),			/* plug'n'pray */
+    (CS8 | CREAD | CLOCAL | HUPCL),			/* Wacom Graphire tablet/mouse */
 };
 
 static const unsigned char proto[][5] =
@@ -99,6 +101,9 @@ static const unsigned char proto[][5] =
     {0, 0, 0, 0, 0},			/* Dummy entry for MOUSE_NONE */
     {0xc0, 0x40, 0xc0, 0x00, 4},	/* IntelliMouse (Serial) */
     {0xc8, 0x08, 0x00, 0x00, 4},	/* IntelliMouse (PS/2) */
+    {0x40, 0x40, 0x40, 0x00, 3},	/* pnp */
+    {0x80, 0x80, 0x80, 0x00, 7},	/* Wacom Graphire */
+    {0xc8, 0x08, 0x00, 0x00, 4},	/* DRMOUSE4DS (Digital Research 2-wheel PS/2) */
 };
 
 static void ms_setspeed(const int old, const int new,
@@ -435,7 +440,8 @@ static int ms_init(void)
     if ((__svgalib_mouse_fd = open(m_dev, O_RDWR | O_NDELAY )) < 0)
 	return -1;
 
-    if (m_type == MOUSE_BUSMOUSE || m_type == MOUSE_PS2 || m_type == MOUSE_IMPS2 || m_type == MOUSE_GPM)
+    if (m_type == MOUSE_BUSMOUSE || m_type == MOUSE_PS2 
+    		|| m_type == MOUSE_IMPS2 || m_type == MOUSE_GPM || m_type == MOUSE_DRMOUSE4DS)
 	m_modem_ctl = 0;
 
     /* If no signal will change there is no need to restore
@@ -479,9 +485,37 @@ static int ms_init(void)
 	ms_setspeed(1200, 1200, cflag[m_type]);
 	write(__svgalib_mouse_fd, "*X", 2);
 	ms_setspeed(1200, m_baud, cflag[m_type]);
-    } else if (m_type == MOUSE_IMPS2) {
+    } else if (m_type == MOUSE_WACOM_GRAPHIRE) {
+    	m_baud = 9600;
+    	ms_setspeed(1200, m_baud, cflag[m_type]);
+    	/* Reset baud rate */
+    	write(__svgalib_mouse_fd, "\r$", 2);
+    	usleep(250000);
+    	/* Reset tablet */
+    	write(__svgalib_mouse_fd, "\r#", 2);
+    	usleep(75000);
+    	/* Set hardware filtering */
+    	write(__svgalib_mouse_fd, "\rSU3", 4);
+    	usleep(75000);
+    	/* Start sending coordinates */
+    	write(__svgalib_mouse_fd, "\rST\r", 4);
+    } else if (m_type == MOUSE_IMPS2 || m_type == MOUSE_DRMOUSE4DS) {
 	/* Initialize the mouse into wheel mode */
 	write(__svgalib_mouse_fd, "\363\310\363\144\363\120", 6);
+    } else if (m_type == MOUSE_PNP) {
+    	/* Need to do this termios stuff here, by hand, ms_setspeed won't 
+	   work with pnp */
+	struct termios tty;
+ 	m_baud = 1200;
+	tcgetattr(__svgalib_mouse_fd, &tty);   
+	tty.c_iflag = IGNBRK | IGNPAR;
+	tty.c_oflag = 0;
+	tty.c_lflag = 0;
+	tty.c_line = 0;
+	tty.c_cc[VTIME] = 0;
+	tty.c_cc[VMIN] = 1;
+	tty.c_cflag = cflag[m_type] | B1200;
+	tcsetattr(__svgalib_mouse_fd, TCSAFLUSH, &tty);
     } else if (m_type != MOUSE_BUSMOUSE && m_type != MOUSE_PS2) {
 	ms_setspeed(9600, m_baud, cflag[m_type]);
 	ms_setspeed(4800, m_baud, cflag[m_type]);
@@ -526,9 +560,17 @@ static int ms_init(void)
             MOUSE_RESETBUTTON;
         mouse_caps.axes = MOUSE_6DIM;
         break;
-        
+    
+    case MOUSE_DRMOUSE4DS:
+    	/* X, Y, RX and RY (two wheels), 3 buttons */
+	mouse_caps.axes = MOUSE_2DIM | MOUSE_RXDIM | MOUSE_RYDIM;
+	mouse_caps.buttons = MOUSE_LEFTBUTTON | MOUSE_RIGHTBUTTON | MOUSE_MIDDLEBUTTON;
+	mouse_caps.info = MOUSE_INFO_WHEEL;
+	break;
+	
     case MOUSE_INTELLIMOUSE:
     case MOUSE_IMPS2:
+    case MOUSE_WACOM_GRAPHIRE:
         /* X, Y, RX (wheel), 3 buttons, wheel */
         mouse_caps.axes = MOUSE_RXDIM;
         mouse_caps.info = MOUSE_INFO_WHEEL;
@@ -595,6 +637,8 @@ static int get_ms_event(int wait) {
     static int but = 0;		/* static is hack for MouseMan */
     static int mouse_orientation = 0;
     int dx=0, dy=0, dz=0, drx=0, dry=0, drz=0;
+    int ax=0, ay=0;
+    static int oldax=0, olday=0, nodev = 0;
     int j;
     char SpaceWare[] = "SpaceWare!";
 
@@ -706,7 +750,8 @@ static int get_ms_event(int wait) {
 	goto handle_packets;
     }
     /* Check whether it's a valid data packet. */
-    if ((m_type != MOUSE_PS2)&&(m_type != MOUSE_IMPS2)&&(m_type != MOUSE_SPACEBALL))
+    if ((m_type != MOUSE_PS2)&&(m_type != MOUSE_IMPS2)&&(m_type != MOUSE_SPACEBALL)
+    	&&(m_type != MOUSE_WACOM_GRAPHIRE) && (m_type != MOUSE_DRMOUSE4DS) )
 	for (j = 1; j < proto[m_type][4]; j++)
 	    if ((buf[i + j] & proto[m_type][2]) != proto[m_type][3]
 		|| buf[i + j] == 0x80) {
@@ -717,11 +762,85 @@ static int get_ms_event(int wait) {
     switch (m_type) {
     case MOUSE_MICROSOFT:	/* Microsoft */
     case MOUSE_LOGIMAN:	/* MouseMan / TrackMan */
+    case MOUSE_PNP:
     default:
 	but = (but & 0x0A) | ((buf[i] & 0x20) >> 3) | ((buf[i] & 0x10) >> 4);
 	dx = (char) (((buf[i] & 0x03) << 6) | (buf[i + 1] & 0x3F));
 	dy = (char) (((buf[i] & 0x0C) << 4) | (buf[i + 2] & 0x3F));
         break;
+    case MOUSE_WACOM_GRAPHIRE: /* Wacom Graphire Tablet */
+        if (!(buf[i] & 0x40)) { /* no device on tablet */
+        	nodev = 1;
+        	break;
+        }
+        but =   (buf[i+3] & 0x08) ? MOUSE_LEFTBUTTON : 0 |
+        	(buf[i+3] & 0x10) ? MOUSE_RIGHTBUTTON : 0 |
+        	(buf[i+3] & 0x20) ? MOUSE_MIDDLEBUTTON : 0;
+
+        /* The absolute position is returned, not the change in position, so
+           we convert it. */
+        ax = ((buf[i+0] & 0x03) << 14) | (buf[i+1] << 7) | buf[i+2];
+        ay = ((buf[i+3] & 0x03) << 14) | (buf[i+4] << 7) | buf[i+5];
+
+        if (nodev) {
+        	oldax = ax;
+        	olday = ay;
+        	nodev = 0;
+        }
+
+	dx = ax - oldax;
+	dy = ay - olday;
+
+	dz = (((buf[i+6] & 0x3f) << 1) | ((buf[i+3] & 0x04) >> 2));
+
+	if (buf[i+6] & 0x40)
+		dz = -dz;
+
+	/* The tablet has *very* high resolution, so we normalize
+	   that a bit. */
+	dx /= 2;
+	dy /= 2;
+	
+	oldax = ax;
+	olday = ay;
+
+	if (buf[i] & 0x20 && dz) /* stylus has pressure */
+		but |= MOUSE_LEFTBUTTON;
+	else if (buf[i+6] & 0x30) { /* roller is being turned */
+		wheel = (buf[i+6] & 0x30) >> 3;
+		if (buf[i+6] & 0x40)
+			wheel = -wheel;
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
+#endif
+            /* RX-axis */
+            if(m_wheel_delta) {
+                drx = ((wheel < 0) ? (-m_wheel_delta) : m_wheel_delta);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; RX axis delta = %d", drx);
+#endif
+            }
+
+            /* Are fake keyboard events enabled? */
+            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
+                /* Fake keypresses... */
+                int key = ((wheel < 0) ? m_fake_upscancode : m_fake_downscancode);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; pressing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 1); /* press */
+                /* todo: delay */
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, ", releasing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 0); /* release */
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, ".\n");
+#endif
+            }
+        }
+	break;
+
     case MOUSE_INTELLIMOUSE:    /* Serial IntelliMouse */
         /* This bit modified from gpm 1.13 */
         but = ((buf[i] & 0x20) >> 3)         /* left */
@@ -788,6 +907,58 @@ static int get_ms_event(int wait) {
 	dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
 	dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
 	break;
+    case MOUSE_DRMOUSE4DS:
+    /* Digital Research 4-Axis mouse - like the PS/2 IntelliMouse, but 
+       has two wheels.  */
+        /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
+        but = ((buf[i] & 1) << 2) /* left */
+            | ((buf[i] & 6) >> 1); /* middle and right */
+        dx = (buf[i] & 0x10) ? buf[i + 1] - 256 : buf[i + 1];
+        dy = (buf[i] & 0x20) ? -(buf[i + 2] - 256) : -buf[i + 2];
+
+        /* Did we turn the wheel? */
+        if((wheel = buf[i + 3]) != 0) {
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, " Wheel turned (0x%02x)", wheel);
+#endif
+	/* RX reports as 1 or F, RY reports as 2 or E */
+	/* both never report at the same time, which makes life easier */
+	if (m_wheel_delta) 
+	{
+		switch (wheel & 0x0F)
+		{
+		case 1:		drx = m_wheel_delta; break;
+		case 15:	drx = -m_wheel_delta; break;
+		case 2:		dry = -m_wheel_delta; break;
+		case 14:	dry = m_wheel_delta; break;
+		};
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; RX axis delta = %d : RY axis delta = %d", drx, dry);
+#endif
+
+            /* Are fake keyboard events enabled? */
+            if(m_fake_kbd_events && __svgalib_keyboard_eventhandler) {
+                /* Fake keypresses... */
+                int key = ((drx+dry < 0) ? m_fake_upscancode : m_fake_downscancode);
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, "; pressing scancode %d", key);
+#endif
+                 __svgalib_keyboard_eventhandler(key, 1); /* press */
+                /* todo: delay */
+#ifdef DEBUG_WHEEL
+                fprintf(stderr, ", releasing scancode %d", key);
+#endif
+                __svgalib_keyboard_eventhandler(key, 0); /* release */
+            }
+#ifdef DEBUG_WHEEL
+            fprintf(stderr, ".\n");
+#endif
+
+
+	}
+    }
+    break;
+    
     case MOUSE_IMPS2:           /* PS/2 IntelliMouse */
         /* This bit modified from the gpm 1.13 imps2 patch by Tim Goodwin */
         but = ((buf[i] & 1) << 2) /* left */
