@@ -21,6 +21,10 @@ extern "C"
 #define MOUSE_NONE 9 		/* Some special number for a non supported/existing mouse */
 #define MOUSE_INTELLIMOUSE 10
 #define MOUSE_IMPS2 11
+#define MOUSE_PNP 12
+#define MOUSE_WACOM_GRAPHIRE 13
+#define MOUSE_DRMOUSE4DS 14
+#define MOUSE_LAST MOUSE_DRMOUSE4DS
 
 #define MOUSE_ORIENTATION_VERTICAL 0
 #define MOUSE_ORIENTATION_HORIZONTAL 1
