@@ -61,7 +61,7 @@ static int nv3_is_linear;
 static CardSpecs *cardspecs;
 static int PLL_INPUT_FREQ;
 
-static unsigned long MMIOBASE, LINEARBASE;
+static unsigned long MMIOBASE=0, LINEARBASE;
 
 enum {
    Riva128 = 0, RivaTNT, GEFORCE
@@ -114,7 +114,6 @@ static int nv3_saveregs(unsigned char regs[])
     regs[NV3REG_SAVE(6)] = PCRTC_Read(FIFO);
     regs[NV3REG_SAVE(7)] = 0;
 
-
    k = PFB_Read(CONFIG_0) ;
    regs[NV3REG_SAVE(8)] = k&0xff;
    regs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
@@ -156,6 +155,8 @@ static void nv3_setregs(const unsigned char regs[], int mode)
 /*  PCRTC_Write(FIFO,regs[NV3REG_SAVE(6)]); */
 
    __svgalib_outcrtc(0x1c,regs[88]); /* this enables banking at 0xa0000 */
+   __svgalib_outcrtc(0x1d,regs[89]);
+   __svgalib_outcrtc(0x1e,regs[90]);
    __svgalib_outcrtc(0x31,0); /* disable cursor */
 
    k=regs[NV3REG_SAVE(8)]+(regs[NV3REG_SAVE(9)] << 8)+(regs[NV3REG_SAVE(10)] << 16)+(regs[NV3REG_SAVE(11)] << 24); 
@@ -177,10 +178,6 @@ static int nv3_modeavailable(int mode)
     struct info *info;
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
-
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
-    if(modeinfo->bitsPerPixel==24)return 0;
     
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
@@ -192,8 +189,11 @@ static int nv3_modeavailable(int mode)
     
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
    
-/*    if(modeinfo->greenWeight==5)return 0; */
-
+    if(modeinfo->bitsPerPixel==24) {
+	free(modeinfo);
+        return 0;
+    }
+    
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
 	free(modetiming);
@@ -332,6 +332,7 @@ static void nv3_initializemode(unsigned char *moderegs,
                             PRAMDAC_Def(PLL_COEFF_SELECT_VCLK_RATIO,DB2);
           break;
       case RivaTNT:
+      case GEFORCE:
           k=0x10000700;
           break;
    };
@@ -345,15 +346,16 @@ static void nv3_initializemode(unsigned char *moderegs,
    * give some significant performance gains. Will perhaps experiment 
    * later on with this stuff!  
    */
-   k=
-      PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
-      PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
-      PFB_Def(CONFIG_0_TILING,ENABLED); 
-   moderegs[NV3REG_SAVE(8)] = k&0xff;
-   moderegs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
-   moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
-   moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
-
+   if(nv3_chiptype<GEFORCE) {
+       k=
+           PFB_Val(CONFIG_0_RESOLUTION,((modeinfo->lineWidth+31)/32))|
+           PFB_Val(CONFIG_0_PIXEL_DEPTH,pixelDepth)|
+           PFB_Def(CONFIG_0_TILING,ENABLED); 
+       moderegs[NV3REG_SAVE(8)] = k&0xff;
+       moderegs[NV3REG_SAVE(9)] = (k >> 8)&0xff;
+       moderegs[NV3REG_SAVE(10)] = (k >> 16) & 0xff ;
+       moderegs[NV3REG_SAVE(11)] = ( k >> 24 ) & 0xff ;
+   }
   moderegs[88]=28;
 
   if(modeinfo->greenWeight==5){moderegs[NV3REG_SAVE(21)]&=0xef;}; 
@@ -372,7 +374,28 @@ static int nv3_setmode(int mode, int prv_mode)
     int i;
     if ((mode < G640x480x256 /*&& mode != G320x200x256*/)
 	|| mode == G720x348x2) {
-        if(nv3_chiptype==Riva128)PRAMDAC_Write(PLL_COEFF_SELECT,0x00000100);
+
+	unsigned int k;
+
+        if(nv3_chiptype==Riva128)
+            PRAMDAC_Write(PLL_COEFF_SELECT,0x00000100);
+            else PRAMDAC_Write(PLL_COEFF_SELECT,0x00000500);
+
+        __svgalib_outcrtc(NV_PCRTC_REPAINT0,0);
+        __svgalib_outcrtc(NV_PCRTC_REPAINT1,0x3d);
+        __svgalib_outcrtc(NV_PCRTC_EXTRA,0);
+        __svgalib_outcrtc(NV_PCRTC_PIXEL,0);
+        __svgalib_outcrtc(NV_PCRTC_HORIZ_EXTRA,0);
+        __svgalib_outcrtc(NV_PCRTC_FIFO_CONTROL,0x83);
+        __svgalib_outcrtc(0x1c,0x18);
+        __svgalib_outcrtc(0x1d,0);
+        __svgalib_outcrtc(0x1e,0);
+        __svgalib_outcrtc(0x30,0);
+        __svgalib_outcrtc(0x31,0);
+        k =  PRAMDAC_Read(GENERAL_CONTROL);
+        k &= ~0x00100000;
+        PRAMDAC_Write(GENERAL_CONTROL,k);
+
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
     }
 
@@ -450,28 +473,22 @@ static int nv3_test(void)
       if(found)return 0;
    };
    if (_ioperm) iopl(0);
-   if ((((buf[0]>>16)&0xffff)!=0x18)&&
-       (((buf[0]>>16)&0xffff)!=0x20)&&
-       (((buf[0]>>16)&0xffff)!=0x29)&&
-       (((buf[0]>>16)&0xffff)!=0x2C)&&
-       (((buf[0]>>16)&0xffff)!=0x2D)&&
-       (((buf[0]>>16)&0xffff)!=0xA0)&&
-       (((buf[0]>>16)&0xffff)!=0x28)&&
-       (((buf[0]>>16)&0xffff)!=0x100)&&
-       (((buf[0]>>16)&0xffff)!=0x101)&&
-       (((buf[0]>>16)&0xffff)!=0x103)
-       )return 0;
    switch(buf[0]>>16){
       case 0x18: nv3_chiptype=Riva128; break;
       case 0x20:
-      case 0x29: case 0x2C: case 0x2D: case 0xA0:
+      case 0x29: case 0x2A: case 0x2B: case 0x2C: case 0x2D: case 0x2E: case 0x2F:
+      case 0xA0:
       case 0x28: nv3_chiptype=RivaTNT; break;
+      case 0x150: case 0x151: case 0x152: case 0x153:
+      case 0x110: case 0x111: case 0x113:
       case 0x101: case 0x103:
       case 0x100: nv3_chiptype=GEFORCE; break;
-      default: nv3_chiptype=Riva128;
+      default: nv3_chiptype=-1;
    };
-   MMIOBASE=buf[4]&0xffffff00;
-   LINEARBASE=buf[5]&0xffffff00;
+
+   if(nv3_chiptype==-1)return 0;
+   MMIOBASE=0;
+   LINEARBASE=0;
    nv3_init(0,0,0);
 return 1;
 }
@@ -585,19 +602,21 @@ static int nv3_init(int force, int par1, int par2)
            exit(1);
          }
        }
+       
        found=__svgalib_pci_find_vendor_vga(0x12d2,buf,0);
-       if (_ioperm) iopl(0);
-       if (found)return 0;
        if (found) {
           found=__svgalib_pci_find_vendor_vga(0x10de,buf,0);
           if (_ioperm) iopl(0);
           if(found)return 0;
-       };
+       } else if (_ioperm) iopl(0);
        switch(buf[0]>>16){
           case 0x18: nv3_chiptype=Riva128; break;
           case 0x20:
-          case 0x29: case 0x2C: case 0x2D: case 0xA0:
+          case 0x29: case 0x2A: case 0x2B: case 0x2C: case 0x2D: case 0x2E: case 0x2F:
+          case 0xA0:
           case 0x28: nv3_chiptype=RivaTNT; break;
+          case 0x150: case 0x151: case 0x152: case 0x153:
+          case 0x110: case 0x111: case 0x113:
           case 0x101: case 0x103:
           case 0x100: nv3_chiptype=GEFORCE; break;
           default: nv3_chiptype=Riva128;
@@ -627,7 +646,7 @@ static int nv3_init(int force, int par1, int par2)
                  if(nv3_memory==2048)nv3_memory=32768;
                  break;
           case GEFORCE:
-             	 nv3_memory=8192; /* do this later */
+             	 nv3_memory=16384; /* do this later */
                  break;
        };
     };
@@ -654,13 +673,13 @@ static int nv3_init(int force, int par1, int par2)
 
     cardspecs = malloc(sizeof(CardSpecs));
     cardspecs->videoMemory = nv3_memory;
-    cardspecs->maxPixelClock4bpp = 75000;	
-    cardspecs->maxPixelClock8bpp = 230000;	
-    cardspecs->maxPixelClock16bpp = 230000;	
-    cardspecs->maxPixelClock24bpp = 230000;
+    cardspecs->maxPixelClock4bpp = 75000;
+    cardspecs->maxPixelClock8bpp = 230000;
+    cardspecs->maxPixelClock16bpp = 230000;
+    cardspecs->maxPixelClock24bpp = 0;
     cardspecs->maxPixelClock32bpp = 230000;
     cardspecs->flags = CLOCK_PROGRAMMABLE ;
-    cardspecs->maxHorizontalCrtc = 2040;
+    cardspecs->maxHorizontalCrtc = 4088;
     cardspecs->maxPixelClock4bpp = 0;
     cardspecs->nClocks =0;
     cardspecs->clocks = NULL;
@@ -669,7 +688,7 @@ static int nv3_init(int force, int par1, int par2)
     cardspecs->matchProgrammableClock=nv3_match_programmable_clock;
     __svgalib_driverspecs = &__svgalib_nv3_driverspecs;
     
-    if((nv3_chiptype==RivaTNT)||(nv3_chiptype=GEFORCE))
+    if((nv3_chiptype==RivaTNT)||(nv3_chiptype==GEFORCE))
       __svgalib_driverspecs->unlock=nv4_unlock;
     
     __svgalib_banked_mem_base=0xa0000;
