@@ -44,7 +44,6 @@
 #include "vga.h"
 #include "libvga.h"
 #include "driver.h"
-#include <sys/mman.h>
 #include <sys/types.h>
 #include "oak.regs"
 
@@ -101,7 +100,6 @@ static int oak_chiptype;
 static int oak_memory;
 static int oak_lockreg_save;
 static unsigned char last_page = 0;
-static void *linearframebuffer;
 static unsigned char *VGAMemory_Save;
 
 static int oak_init(int, int, int);
@@ -117,6 +115,7 @@ static void oak_clearlinearmode(void);
 static void oak_setpage(int page);
 static void OAK_WriteDAC(int function);
 static void OAKSetByte(int, int);
+static void * linearframebuffer;
 
 /* Mode table */
 
@@ -232,15 +231,15 @@ static void oak_setregs(const unsigned char regs[], int mode)
 	OAKSetByte(OTI87_XREADSEGMENT, regs[OTI87_XREADSEGMENT_DATA]);
 	OAKSetByte(OTI87_XWRITESEGMENT, regs[OTI87_XWRITESEGMENT_DATA]);
 
-	outb(0x3c4, 0);		/* reset sync. */
-	junk = inb(0x3c5);
-	outw(0x3c4, 0x00 + ((junk & 0xfd) << 8));
+	outb(SEQ_I, 0);		/* reset sync. */
+	junk = inb(SEQ_D);
+	outw(SEQ_I, 0x00 + ((junk & 0xfd) << 8));
 
 	OAKSetByte(OTI87_CLOCK, regs[EXT + 8]);
 	OAKSetByte(OTI87_FIFO, regs[OTI87_FIFO_DATA]);
 	OAKSetByte(OTI87_MODESELECT, regs[OTI87_MODESELECT_DATA]);
 
-	outw(0x3c4, 0x00 + (junk << 8));	/* set sync. */
+	outw(SEQ_I, 0x00 + (junk << 8));	/* set sync. */
 
 	OAKSetByte(OTI87_XCRTC, regs[OTI87_XCRTC_DATA]);
 	OAKSetByte(OTI87_OVERFLOW, regs[EXT + 9]);
@@ -250,14 +249,14 @@ static void oak_setregs(const unsigned char regs[], int mode)
 	OAKSetByte(OTI_SEGMENT, regs[OTI_SEGMENT_DATA]);
 
 	/* doc says don't OTI-MISC unless sync. reset is off */
-	outb(0x3c4, 0);
-	junk = inb(0x3c5);
-	outw(0x3C4, 0x00 + ((junk & 0xFD) << 8));	/* now disable the timing sequencer */
+	outb(SEQ_I, 0);
+	junk = inb(SEQ_D);
+	outw(SEQ_I, 0x00 + ((junk & 0xFD) << 8));	/* now disable the timing sequencer */
 
 	OAKSetByte(OTI_MISC, regs[OTI_MISC_DATA]);
 
 	/* put sequencer back */
-	outw(0x3C4, 0x00 + (junk << 8));
+	outw(SEQ_I, 0x00 + (junk << 8));
 
 	OAKSetByte(OTI_BCOMPAT, regs[OTI_BCOMPAT_DATA]);
 	OAKSetByte(OTI_CONFIG, regs[OTI_CONFIG_DATA]);
@@ -501,7 +500,7 @@ static void oak_setwrpage(int page)
 /* oak_setdisplaystart( int address )                   Set display address
 
  * Sets display start address.
- * First word goes into 0x3d4 indices 0x0c and 0x0d
+ * First word goes into CRT_IC indices 0x0c and 0x0d
  * If 067, bit 16 goes to OTI_OVERFLOW bit 3
  * If 077, bit 17 goes to OTI_OVERFLOW2 bit 3
  * If 087, bits 16, 17, and 18 go to OTI87_XCRTC bits 0-2
@@ -509,11 +508,11 @@ static void oak_setwrpage(int page)
 
 static void oak_setdisplaystart(int address)
 {
-    outw(0x3d4, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
-    outw(0x3d4, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
+    outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
+    outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     inb(0x3da);			/* set ATC to addressing mode */
-    outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+    outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
+    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
 
     if (oak_chiptype == 87) {
@@ -539,7 +538,7 @@ static void oak_setdisplaystart(int address)
 
 static void oak_setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
 }
 
 /* oak_init ( int force, int par1, int par2)            Initialize chipset
@@ -565,6 +564,12 @@ static int oak_init(int force, int par1, int par2)
 	       oak_memory);
     }
     __svgalib_driverspecs = &__svgalib_oak_driverspecs;
+
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+    __svgalib_linear_mem_base=0xe00000;
+    __svgalib_linear_mem_size=oak_memory*0x400;
+
     return 1;
 }
 
@@ -635,7 +640,7 @@ static int oak_chipdetect(void)
     /* none of the above ... maybe the nonexistant '97 I've been 
        hearing about? */
 
-    printf("Oak driver: Unknown chipset (id = %2x)\n", temp1);
+/*    printf("Oak driver: Unknown chipset (id = %2x)\n", temp1);*/
     oak_lock();
     return 0;
 }
@@ -668,15 +673,15 @@ static void oak_setlinearmode(void)
     int temp1;
     temp1 = ((OAKGetByte(OTI87_STATUS) & 0x06) << 1) | 0xe1;
     OAKSetByte(OTI87_VIDMAP, temp1);
-    linearframebuffer = (unsigned char *) mmap((caddr_t) 0x40000000,
+/*    linearframebuffer = (unsigned char *) mmap((caddr_t) 0x40000000,
 					       oak_memory * 1024,
 					       PROT_READ | PROT_WRITE,
 					       MAP_SHARED | MAP_FIXED,
 					       __svgalib_mem_fd,
 					       0xe00000);
+*/
+    linearframebuffer=LINEAR_MEM_POINTER;
     VGAMemory_Save = __svgalib_graph_mem;
-/*      __svgalib_graph_mem = linearframebuffer; */
-/*      graph_mem = __svgalib_graph_mem; */
     __svgalib_modeinfo_linearset |= IS_LINEAR;
 }
 
@@ -692,7 +697,6 @@ static void oak_clearlinearmode(void)
     __svgalib_modeinfo_linearset ^= IS_LINEAR;
     __svgalib_graph_mem = VGAMemory_Save;
     graph_mem = VGAMemory_Save;
-    munmap((caddr_t) 0x40000000, oak_memory * 1024);
 
 }
 
