@@ -414,7 +414,7 @@ typedef struct {
         unsigned char extdac[16];
         int ics2595;
        } ATIHWRec, *ATIHWPtr;
-
+extern unsigned char __svgalib_ragedoubleclock;
 static int rage_init(int, int, int);
 static void rage_unlock(void);
 static void rage_lock(void);
@@ -866,6 +866,9 @@ static void rage_setregs(const unsigned char regs[], int mode)
           ATIPutMach64PLLReg(PLL_VCLK_CNTL,i);
        break;
     };
+
+    /* make sure the dac is in 8 bit or 6 bit mode, as needed */
+    outl(ATIIOPortDAC_CNTL, restore->dac_cntl);
  
     outb(ATIIOPortDAC_MASK, 0xFFU);
     outb(ATIIOPortDAC_WRITE, 0x00U);
@@ -897,8 +900,6 @@ static int rage_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -906,6 +907,7 @@ static int rage_modeavailable(int mode)
     info = &__svgalib_infotable[mode];
     if (rage_memory * 1024 < info->ydim * info->xbytes)
 	return 0;
+
     modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
     modetiming = malloc(sizeof(ModeTiming));
     if (__svgalib_getmodetiming(modetiming, modeinfo, cardspecs)) {
@@ -984,13 +986,13 @@ static void rage_initializemode(unsigned char *moderegs,
     else
         ATINewHWPtr->bus_cntl |= BUS_APER_REG_DIS;
 
+    ATINewHWPtr->dac_cntl=inl(ATIIOPortDAC_CNTL);
+
     if (modeinfo->bitsPerPixel>8)
             ATINewHWPtr->dac_cntl |= DAC_8BIT_EN;
       else
             ATINewHWPtr->dac_cntl &= ~DAC_8BIT_EN;
 
-    ATINewHWPtr->dac_cntl=inl(ATIIOPortDAC_CNTL);
-
     ATINewHWPtr->config_cntl= inl(ATIIOPortCONFIG_CNTL) | CFG_MEM_VGA_AP_EN;
 
     ATINewHWPtr->ovr_clr=0;
@@ -1050,9 +1052,6 @@ static void rage_initializemode(unsigned char *moderegs,
     ATINewHWPtr->crtc_gen_cntl |= CRTC_EXT_DISP_EN | CRTC_EN |
           CRTC_VGA_LINEAR | CRTC_CNT_EN | CRTC_BYTE_PIX_ORDER;
 
-    if(modeinfo->bitsPerPixel>8)ATINewHWPtr->dac_cntl |= DAC_8BIT_EN; else
-       ATINewHWPtr->dac_cntl &= ~DAC_8BIT_EN;
-
     switch (modeinfo->bitsPerPixel)
         {
             case 1:
@@ -1109,8 +1108,6 @@ static void rage_initializemode(unsigned char *moderegs,
        break;       
     };
 
-
-
     rage_bpp = modeinfo->bytesPerPixel; 
 
 if((ATIChip>=ATI_CHIP_264VTB)&&(ATIIODecoding==BLOCK_IO))
@@ -1541,6 +1538,7 @@ static int rage_init(int force, int par1, int par2)
       maxN=255;
       fref=28636;
       Nadj=0;
+      if(__svgalib_ragedoubleclock)fref*=2;
    } else {
       rage_dac=(j>>9)&7;
       if(rage_dac==5) {
@@ -1594,7 +1592,8 @@ static int rage_init(int force, int par1, int par2)
          cardspecs->maxPixelClock32bpp = 40000;
    };
    cardspecs->flags = CLOCK_PROGRAMMABLE;
-   cardspecs->maxHorizontalCrtc = 2040;
+   cardspecs->maxHorizontalCrtc = 4088;
+   cardspecs->nClocks = 0;
    cardspecs->maxPixelClock4bpp = 0;
    cardspecs->mapClock = rage_map_clock;
    cardspecs->mapHorizontalCrtc = rage_map_horizontal_crtc;
@@ -1605,13 +1604,52 @@ static int rage_init(int force, int par1, int par2)
    __svgalib_linear_mem_base=rage_linear_base;
    __svgalib_linear_mem_size=rage_memory*0x400;
 
+#define BIOSWord(x) ((int)BIOS[x]+256*(int)BIOS[x+1])
+
    BIOS=mmap(0,32*1024,PROT_READ|PROT_WRITE,MAP_SHARED,__svgalib_mem_fd,0xc0000);
 
-   i=(int)BIOS[0x48]+256*(int)BIOS[0x49];
-   j=(int)BIOS[i+0x10]+256*(int)BIOS[i+0x11];
+   i=BIOSWord(0x48);
+   j=BIOSWord(i+0x10);
 
    ATIClockToProgram=BIOS[j+6];
 
+   if(ATIChip>=ATI_CHIP_264CT){
+      M=ATIGetMach64PLLReg(PLL_REF_DIV);
+      minN=2;
+      maxN=255;
+      switch (BIOSWord(j+0x08)/10) {
+         case 143:
+            fref=14318; 
+            break;
+         case 286:
+            fref=28636;
+            break;
+         default:
+            fref=BIOSWord(j+0x08)*10;
+            break;
+      }
+      Nadj=0;
+      fref*=2; /* X says double for all chips */
+#if 0
+      if(__svgalib_ragedoubleclock)fref/=2; /* just in case */
+#endif
+      if (__svgalib_driver_report) {
+          printf("Rage: BIOS reports base frequency=%.3fMHz  Denominator=%3i\n",fref/1000,M);
+      }
+   } else {
+      rage_dac=(j>>9)&7;
+      if(rage_dac==5) {
+         rage_clock=1;
+         M=46;
+         minN=257;
+         maxN=512;
+         Nadj=257;
+         for(i=0;i<4;i++)postdiv[i]=1<<i;
+         for(i=4;i<8;i++)postdiv[i]=0;
+         fref=14318;
+      };
+   };
+   
    munmap(BIOS,32768);
 
    return 0;
