@@ -1095,12 +1095,14 @@ static int s3_setmode(int mode, int prv_mode)
     if (mode < G640x480x256 || mode == G720x348x2) {
 	/* Let the standard VGA driver set standard VGA modes. */
 	res = __svgalib_vga_driverspecs.setmode(mode, prv_mode);
-	if (res == 0 && s3_chiptype <= S3_928) {
+	if (res == 0) {
 	    /*
 	     * ARI: Turn off virtual size of 1024 - this fixes all problems
 	     *      with standard modes, including 320x200x256.
 	     * 
 	     * SL:  Is this for 928 only?  Doesn't matter for 805.
+             *
+             * MZ:  Affects 765 as well, so I assume it is good for all chipsets.
 	     */
 	    s3_unlock();
 	    __svgalib_outCR(0x34, __svgalib_inCR(0x34) & ~0x10);
