@@ -37,7 +37,7 @@ chipset companies still do this).
 #include "vgapci.h"
 
 #define SISREG_SAVE(i) (VGA_TOTAL_REGS+i)
-#define SIS_TOTAL_REGS (VGA_TOTAL_REGS + 64)
+#define SIS_TOTAL_REGS (VGA_TOTAL_REGS + 0x48 - 5)
 
 #define XR(i) (VGA_TOTAL_REGS+i-5)
 
@@ -50,6 +50,9 @@ chipset companies still do this).
 #define PCI_CHIP_SIS5597        0x0200
 #define PCI_CHIP_SIS6326        0x6326
 #define PCI_CHIP_SIS530        0x6306
+#define PCI_CHIP_SIS300         0x0300
+#define PCI_CHIP_SIS540         0x5300
+#define PCI_CHIP_SIS630         0x6300
 
 #define ClockReg	XR(0x07) 
 #define DualBanks	XR(0x0B)
@@ -85,8 +88,14 @@ static CardSpecs *cardspecs;
 
 static void sis_setpage(int page)
 {
-        outb(0x3cb,page);
-        outb(0x3cd,page);
+        char tmp;
+
+        read_xr(0x0b,tmp);
+        if (tmp&8) {
+            outb(0x3cb,page);
+            outb(0x3cd,page);
+        } else
+            outb(0x3cd,page|(page<<4));
 }
 
 static int __svgalib_sis_inlinearmode(void)
@@ -120,18 +129,38 @@ static int sis_saveregs(unsigned char regs[])
 { 
     int i;
 
-    sis_unlock();		
-
-    for(i=5; i<0x38;i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+    sis_unlock();
     
+    regs[XR(0x40)] = inb(0x3cb);
+    regs[XR(0x41)] = inb(0x3cd);	
+
     if(sis_chiptype==SIS_5597)
-       for(i=0x38; i<0x3a;i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
- 
+       for(i=0x38; i<0x3a;i++)read_xr(i,regs[XR(i)]);
+
     if(sis_chiptype==SIS_6326)
-       for(i=0x38; i<0x3d;i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+       for(i=0x38; i<0x3d;i++)read_xr(i,regs[XR(i)]);
 
     if(sis_chiptype==SIS_530)
-       for(i=0x38; i<0x40 ; i++)read_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+       for(i=0x38; i<0x40 ; i++)read_xr(i,regs[XR(i)]);
+
+    /* get all the clocks */
+    if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530) {
+       i = regs[XR(0x38)] & 0xfc;
+       write_xr(0x38,i|1);
+       read_xr(0x13,regs[XR(0x42)]);
+       read_xr(0x2a,regs[XR(0x43)]);
+       read_xr(0x2b,regs[XR(0x44)]);
+       write_xr(0x38,i|2);
+       read_xr(0x13,regs[XR(0x45)]);
+       read_xr(0x2a,regs[XR(0x46)]);
+       read_xr(0x2b,regs[XR(0x47)]);
+       write_xr(0x38,i); /* Make sure that the loop gets the internal VCLK registers */
+    }
+
+    for(i=5; i<0x38;i++)read_xr(i,regs[XR(i)]);
+    
+    if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530)
+       write_xr(0x38,regs[XR(0x38)]);
 
     return SIS_TOTAL_REGS - VGA_TOTAL_REGS;
 }
@@ -144,19 +173,33 @@ static void sis_setregs(const unsigned char regs[], int mode)
 
     sis_unlock();
     
-    write_xr(0x5,0x86);
+    outb(0x3cb,regs[XR(0x40)]);
+    outb(0x3cd,regs[XR(0x41)]);
 
-    for(i=6; i<0x38;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+    /* set all the clocks */
+    if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530) {
+       i = regs[XR(0x38)] & 0xfc;
+       write_xr(0x38,i|1);
+       write_xr(0x13,regs[XR(0x42)]);
+       write_xr(0x2a,regs[XR(0x43)]);
+       write_xr(0x2b,regs[XR(0x44)]);
+       write_xr(0x38,i|2);
+       write_xr(0x13,regs[XR(0x45)]);
+       write_xr(0x2a,regs[XR(0x46)]);
+       write_xr(0x2b,regs[XR(0x47)]);
+       write_xr(0x38,i); /* Make sure that the loop puts the internal VCLK registers */
+    }
+
+    for(i=6; i<0x38;i++)write_xr(i,regs[XR(i)]);
     
     if(sis_chiptype==SIS_5597)
-       for(i=0x38; i<0x3a;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+       for(i=0x38; i<0x3a;i++)write_xr(i,regs[XR(i)]);
  
     if(sis_chiptype==SIS_6326)
-       for(i=0x38; i<0x3d;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
+       for(i=0x38; i<0x3d;i++)write_xr(i,regs[XR(i)]);
 
     if(sis_chiptype==SIS_530)
-       for(i=0x38; i<0x40;i++)write_xr(i,regs[VGA_TOTAL_REGS+i-5]);
-
+       for(i=0x38; i<0x40;i++)write_xr(i,regs[XR(i)]);
 }
 
 
@@ -168,8 +211,6 @@ static int sis_modeavailable(int mode)
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
 
-    modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
-
     if ((mode < G640x480x256 )
 	|| mode == G720x348x2)
 	return __svgalib_vga_driverspecs.modeavailable(mode);
@@ -298,7 +339,6 @@ static void sis_initializemode(unsigned char *moderegs,
         }
     }
     
-
     moderegs[59]=0x6f;
 
     return ;
@@ -310,14 +350,30 @@ static int sis_setmode(int mode, int prv_mode)
     unsigned char *moderegs;
     ModeTiming *modetiming;
     ModeInfo *modeinfo;
-    int tmp;
+    int tmp, var;
 
     if ((mode < G640x480x256)||(mode==G720x348x2)) {
         if(sis_chiptype >= SIS_5597) { 
-		/* Work Arrout for vesafb case */
+		/* Work Around for vesafb case */
                 read_xr(0x06,tmp);
                 tmp&=~(0x1E); /* Depth registers */
                 write_xr(0x06,tmp);
+		/* program the 25 and 28 Mhz clocks */
+		if (sis_chiptype==SIS_5597 || sis_chiptype==SIS_6326 || sis_chiptype==SIS_530) {
+                   read_xr(0x38,tmp);
+                   tmp &= 0xfc;
+                   write_xr(0x38,tmp|1);
+	           read_xr(0x13,var);
+                   write_xr(0x13,var|0x40);
+                   write_xr(0x2a,0x1b);
+                   write_xr(0x2b,0xe1);
+                   write_xr(0x38,tmp|2);
+                   read_xr(0x13,var);
+                   write_xr(0x13,var|0x40);
+                   write_xr(0x2a,0x4e);
+                   write_xr(0x2b,0xe4);
+                   write_xr(0x38,tmp);
+		}
         }
  
 	return __svgalib_vga_driverspecs.setmode(mode, prv_mode);
@@ -410,6 +466,9 @@ static int sis_test(void)
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS5597)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS5598)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS530)||
+       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS540)||
+       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS630)||
+       (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS300)||
        (((buf[0]>>16)&0xffff)==PCI_CHIP_SIS6326)
        )){
        sis_init(0,0,0);
@@ -562,6 +621,9 @@ static int sis_init(int force, int par1, int par2)
             case PCI_CHIP_SG86C215: sis_chiptype=SIS_86C215; break;
             case PCI_CHIP_SG86C225: sis_chiptype=SIS_86C225; break;
             case PCI_CHIP_SIS5597: sis_chiptype=SIS_5597; break;
+            case PCI_CHIP_SIS300:
+            case PCI_CHIP_SIS540:
+            case PCI_CHIP_SIS630:
             case PCI_CHIP_SIS530: sis_chiptype=SIS_530; break;
 /*            case PCI_CHIP_SIS5598: sis_chiptype=SIS_5598; break;*/
             case PCI_CHIP_SIS6326: sis_chiptype=SIS_6326; break;
