@@ -247,8 +247,8 @@ static int tvga8900_saveregs(unsigned char regs[])
     port_out(0x0b, SEQ_I);	/* force old mode regs */
     port_out(port_in(SEQ_D), SEQ_D);	/* by writing */
 
-    /* outw(0x3C4, 0x820E); *//* unlock conf. reg */
-    /* port_out(0x0c, SEQ_I); *//* save conf. reg */
+    /* outw(SEQ_I, 0x820E); */ /* unlock conf. reg */
+    /* port_out(0x0c, SEQ_I); */ /* save conf. reg */
     /* regs[EXT + 11] = port_in(SEQ_D); */
 
     port_out(0x0d, SEQ_I);	/* old reg 13 */
@@ -408,8 +408,8 @@ static void tvga8900_setregs(const unsigned char regs[], int mode)
 	port_out(0x0c, SEQ_I);	/* reg 12               */
 	port_out(reg_0c, SEQ_D);
     }
-    /*      outw(0x3c4, 0x820e); *//* unlock conf. reg */
-    /*      port_out(0x0c, SEQ_I); *//* reg 12 */
+    /*      outw(SEQ_I, 0x820e); */ /* unlock conf. reg */
+    /*      port_out(0x0c, SEQ_I); */ /* reg 12 */
 
     port_out(0x0d, SEQ_I);	/* new reg 13 */
     port_out(regs[EXT + 9], SEQ_D);
@@ -436,10 +436,10 @@ static void tvga8900_setregs(const unsigned char regs[], int mode)
     port_out(regs[EXT + 2], SEQ_D);
     port_out(0x0F, SEQ_I);
     port_out(regs[EXT + 3], SEQ_D);
-// you cant write to this anyways... it messes things up...
-//    port_out(0x0B, SEQ_I);
-//    port_out(regs[EXT + 0], SEQ_D);
-
+/*  you cant write to this anyways... it messes things up...
+    port_out(0x0B, SEQ_I);
+    port_out(regs[EXT + 0], SEQ_D);
+*/
     /* write extended CRT registers */
     port_out(0x19, __svgalib_CRT_I);
     port_out(regs[EXT + 4], __svgalib_CRT_D);
@@ -554,20 +554,20 @@ static int tvga8900_test(void)
     /* 
      * Check first that we have a Trident card.
      */
-    outb(0x3c4, 0x0b);
-    save0b = inb(0x3c5);
-    outw(0x3C4, 0x000B);	/* Switch to Old Mode */
-    inb(0x3C5);			/* Now to New Mode */
-    outb(0x3C4, 0x0E);
-    origVal = inb(0x3C5);
-    outb(0x3C5, 0x00);
-    newVal = inb(0x3C5) & 0x0F;
-    outb(0x3C5, (origVal ^ 0x02));
+    outb(SEQ_I, 0x0b);
+    save0b = inb(SEQ_D);
+    outw(SEQ_I, 0x000B);	/* Switch to Old Mode */
+    inb(SEQ_D);			/* Now to New Mode */
+    outb(SEQ_I, 0x0E);
+    origVal = inb(SEQ_D);
+    outb(SEQ_D, 0x00);
+    newVal = inb(SEQ_D) & 0x0F;
+    outb(SEQ_D, (origVal ^ 0x02));
 
     if (newVal != 2) {
-	outb(0x3c5, origVal);
-	outb(0x3c4, 0x0b);
-	outb(0x3c5, save0b);
+	outb(SEQ_D, origVal);
+	outb(SEQ_I, 0x0b);
+	outb(SEQ_D, save0b);
 	return 0;
     }
 
@@ -598,42 +598,42 @@ static void tvga8900_setpage(int page)
 static void tvga8900_setdisplaystart(int address)
 {
     if (__svgalib_cur_mode == G320x200x256) {
-	outw(0x3d4, 0x0d + (address & 0x00ff) * 256);
-	outw(0x3d4, 0x0c + (address & 0xff00));
+	outw(CRT_IC, 0x0d + (address & 0x00ff) * 256);
+	outw(CRT_IC, 0x0c + (address & 0xff00));
 	address <<= 2;		/* Adjust address so that extended bits */
 	/* are correctly set later (too allow for */
 	/* multi-page flipping in 320x200). */
 	goto setextendedbits;
     }
     if (tvga8900_memory == 1024) {
-	outw(0x3d4, 0x0d + ((address >> 3) & 0x00ff) * 256);	/* sa2-sa9 */
-	outw(0x3d4, 0x0c + ((address >> 3) & 0xff00));	/* sa10-sa17 */
+	outw(CRT_IC, 0x0d + ((address >> 3) & 0x00ff) * 256);	/* sa2-sa9 */
+	outw(CRT_IC, 0x0c + ((address >> 3) & 0xff00));	/* sa10-sa17 */
     } else {
-	outw(0x3d4, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
-	outw(0x3d4, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
+	outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
+	outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     }
     if (__svgalib_cur_mode != G320x200x256) {
 	inb(0x3da);		/* set ATC to addressing mode */
-	outb(0x3c0, 0x13 + 0x20);	/* select ATC reg 0x13 */
+	outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
 	if (tvga8900_memory == 1024) {
-	    outb(0x3c0, (inb(0x3c1) & 0xf0) | (address & 7));
+	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 7));
 	    /* write sa0-2 to bits 0-2 */
 	    address >>= 1;
 	} else
-	    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
 	/* write sa0-1 to bits 1-2 */
     }
   setextendedbits:
-    outb(0x3d4, 0x1e);
-    outb(0x3d5, (inb(0x3d5) & 0x5f) | 0x80	/* set bit 7 */
+    outb(CRT_IC, 0x1e);
+    outb(CRT_DC, (inb(CRT_DC) & 0x5f) | 0x80	/* set bit 7 */
 	 | ((address & 0x40000) >> 13));	/* sa18: write to bit 5 */
-    outb(0x3c4, 0x0b);
-    outb(0x3c5, 0);		/* select 'old mode' */
-    outb(0x3c4, 0x0e);
-    outb(0x3c5, (inb(0x3c5) & 0xfe)
+    outb(SEQ_I, 0x0b);
+    outb(SEQ_D, 0);		/* select 'old mode' */
+    outb(SEQ_I, 0x0e);
+    outb(SEQ_D, (inb(SEQ_D) & 0xfe)
 	 | ((address & 0x80000) >> 19));	/* sa19: write to bit 0 */
-    outb(0x3c4, 0x0b);
-    inb(0x3c5);			/* return to 'new mode' */
+    outb(SEQ_I, 0x0b);
+    inb(SEQ_D);			/* return to 'new mode' */
 }
 
 
@@ -642,7 +642,7 @@ static void tvga8900_setdisplaystart(int address)
 
 static void tvga8900_setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
 }
 
 
@@ -718,8 +718,8 @@ bit 0:		force noninterlaced if set
 	tvga8900_nonint = port_in(SEQ_D) & 0x04;
 
 	/* check version */
-	outw(0x3c4, 0x000b);
-	switch (inb(0x3c5)) {
+	outw(SEQ_I, 0x000b);
+	switch (inb(SEQ_D)) {
 	case 0x02:			/* 8800cs */
 	case 0x03:			/* 8900b */
 	case 0x04:			/* 8900c */
@@ -761,5 +761,7 @@ bit 0:		force noninterlaced if set
     }
     __svgalib_driverspecs = &__svgalib_tvga8900_driverspecs;
     setup_registers();
-    return 0;
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
+   return 0;
 }
