@@ -415,18 +415,18 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 	    case G360x480x256:
 	      modeX:
 		{
-		    int first, last, offset, pixel, plane;
-
+		    int first, offset, pixel, plane;
 
 		    for (plane = 0; plane < 4; plane++) {
 			/* select plane */
 			port_out(0x02, SEQ_I);	/* ATI needs to set it */
 			port_out(1 << plane, SEQ_D);
 
-			pixel = plane;
+			pixel = ((4 - (x & 3) + plane) & 3);
 			first = (y * CI.xdim + x) / 4;
-			last = (y * CI.xdim + x + length) / 4;
-			for (offset = first; offset < last; offset++) {
+			if((x & 3) + pixel > 3)
+			    first++;
+			for (offset = first; pixel < length; offset++) {
 			    gr_writeb(colors[pixel], offset);
 			    pixel += 4;
 			}
@@ -699,15 +699,16 @@ int vga_getscansegment(unsigned char *colors, int x, int y, int length)
 	    case G360x480x256:
 	      modeX2:
 		{
-		    int first, last, offset, pixel, plane;
+		    int first, offset, pixel, plane;
 		    for (plane = 0; plane < 4; plane++) {
 			/* select plane */
-			port_out(0x02, SEQ_I);	/* ATI needs resetting that one */
-			port_out(1 << plane, SEQ_D);
-			pixel = plane;
+			port_out(0x04, GRA_I);	/* feel the wrath of the ATI */
+			port_out(plane, GRA_D);
+			pixel = ((4 - (x & 3) + plane) & 3);
 			first = (y * CI.xdim + x) / 4;
-			last = (y * CI.xdim + x + length) / 4;
-			for (offset = first; offset < last; offset++) {
+			if((x & 3) + pixel > 3)
+			    first++;
+			for (offset = first; pixel < length; offset++) {
 			    colors[pixel] = gr_readb(offset);
 			    pixel += 4;
 			}
