@@ -226,26 +226,26 @@ static void setdisplaystart(int address)
 	switch (modeinfo->colors) {
 	case 16:		/* planar 16-color mode */
 	    inb(0x3da);
-	    outb(0x3c0, 0x13 + 0x20);
-	    outb(0x3c0, (inb(0x3c1) & 0xf0) | (address & 7));
+	    outb(ATT_IW, 0x13 + 0x20);
+	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | (address & 7));
 	    /* write sa0-2 to bits 0-2 */
 	    address >>= 3;
 	    break;
 	case 256:		/* planar 256-color mode */
 	    inb(0x3da);
-	    outb(0x3c0, 0x13 + 0x20);
-	    outb(0x3c0, (inb(0x3c1) & 0xf0) | ((address & 3) << 1));
+	    outb(ATT_IW, 0x13 + 0x20);
+	    outb(ATT_IW, (inb(ATT_R) & 0xf0) | ((address & 3) << 1));
 	    /* write sa0-1 to bits 1-2 */
 	    address >>= 2;
 	    break;
 	}
-    outw(0x3d4, 0x0d + (address & 0x00ff) * 256);	/* sa0-sa7 */
-    outw(0x3d4, 0x0c + (address & 0xff00));	/* sa8-sa15 */
+    outw(CRT_IC, 0x0d + (address & 0x00ff) * 256);	/* sa0-sa7 */
+    outw(CRT_IC, 0x0c + (address & 0xff00));	/* sa8-sa15 */
 }
 
 static void setlogicalwidth(int width)
 {
-    outw(0x3d4, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
 }
 
 static int vgadrv_init(int, int, int);
@@ -263,6 +263,8 @@ static int vga_test(void)
     __svgalib_delay();
     outb(PEL_IW, save);
     save = ~save;
+    __svgalib_banked_mem_base=0xa0000;
+    __svgalib_banked_mem_size=0x10000;
     if (back == save) {
 	vgadrv_init(0, 0, 0);
 	return 1;
@@ -295,7 +297,8 @@ DriverSpecs __svgalib_vga_driverspecs =
     0,				/* extset */
     0,
     0,				/* linear */
-    NULL			/* accelspecs */
+    NULL,                       /* Accelspecs */
+    NULL,                       /* Emulation */
 };
 
 /* Initialize chipset (called after detection) */
