@@ -22,9 +22,9 @@
 #include "mouse/vgamouse.h"
 #include "keyboard/vgakeyboard.h"
 
-#ifdef BACKGROUND
+/*#ifdef BACKGROUND*/
 #include "vgabg.h"
-#endif
+/*#endif*/
 
 void vga_waitretrace(void)
 {
@@ -92,57 +92,30 @@ int __svgalib_physmem(void)
 static unsigned char *
  map_framebuffer(unsigned address, int size)
 {
-    int mem_fd;
 #ifndef BACKGROUND
-
-    unsigned char *fb;
-
-    mem_fd = __svgalib_mem_fd;
-    if (mem_fd == -1) {
-	printf("svgalib: memory handle not initialized!\n");
-	exit(-1);
-    }
-    fb = valloc(size);
-    return mmap(
-		   (caddr_t) fb,
-		   size,
-		   PROT_READ | PROT_WRITE,
-		   MAP_FIXED | MAP_SHARED,
-		   mem_fd,
-		   (off_t) address
-	);
-    close(mem_fd); /* Never gets here ? */
-    /* Using addr 0 (kernel comes up with 0x40000000), 
-       and only MAP_SHARED works, but wastes 2Mb 
-       of memory. */
+    return( LINEAR_MEM_POINTER );
 #endif
 #ifdef BACKGROUND
 #if BACKGROUND == 1
  /* Probably won't go right at first try. */
+    int old_modeinf;
 
-    mem_fd = __svgalib_mem_fd;
-    if (mem_fd == -1) {  /* sanity check */
-	printf("svgalib: memory handle not initialized!\n");
-	exit(-1);
-    }
+    if(!__svgalib_linearframebuffer)
     if ((__svgalib_linearframebuffer = valloc(size)) == NULL) {
 	printf("svgalib: allocation error \n");
 	exit(-1);
     }
     __svgalib_linear_memory_size=size;
     __svgalib_graph_mem_linear_check=__svgalib_linearframebuffer;
-    __svgalib_graph_mem_linear_orginal = (unsigned char *) mmap(
-				   (caddr_t) 0,
-				   size,
-				   PROT_READ | PROT_WRITE,
-				   MAP_SHARED,
-				   mem_fd,
-				   (off_t) address
-	);
-	 
+    __svgalib_graph_mem_linear_orginal = (unsigned char *)  LINEAR_MEM_POINTER; 
+
+    old_modeinf=__svgalib_modeinfo_linearset;
+    __svgalib_modeinfo_linearset |= IS_LINEAR; /* fake it for __svgalib_map_virtual_screen() */
+
     __svgalib_map_virtual_screen(VIDEOSCREEN); /* Video page */
-    return (__svgalib_graph_mem_linear_orginal);
-    /* close(mem_rd);  Hmm... seems to be opened many times , never closed */
+    __svgalib_modeinfo_linearset=old_modeinf;
+
+    return (__svgalib_linearframebuffer);
 #endif
 #endif
 }
@@ -179,12 +152,8 @@ static int verify_one_linear_mapping(int base, int size)
 	data = *(unsigned long *) ((unsigned char *) fb + i * 65536);
 	if (data != i) {
 	    munmap((caddr_t) fb, size);
-	    (*lfn) (LINEAR_DISABLE, base);
+            (*lfn) (LINEAR_DISABLE, base);
 	    result = -1;
-#if 0
-	    printf("svgalib: Signature mismatch at 0x%08X (0x%08X), 0x%08X != 0x%08X.\n",
-		   fb + i * 65536, base + i * 65536, data, i);
-#endif
 	    goto cleanupsignatures;
 	}
     }
@@ -192,7 +161,7 @@ static int verify_one_linear_mapping(int base, int size)
     if(__svgalib_driver_report)
     printf("svgalib: Found linear framebuffer at 0x%08X.\n", base);
     result = 0;
-    munmap((caddr_t) fb, size);
+   munmap((caddr_t) fb, size);
     (*lfn) (LINEAR_DISABLE, base);
   cleanupsignatures:
     /* Clean up the signatures (write zeroes). */
@@ -207,6 +176,7 @@ static int verify_one_linear_mapping(int base, int size)
 
 static int verify_linear_mapping(int base, int size)
 {
+    if(__svgalib_linear_mem_size)return __svgalib_linear_mem_size;
     while (size > 256 * 256) {
 	if (verify_one_linear_mapping(base, size) == 0)
 	    return size;
@@ -215,7 +185,6 @@ static int verify_linear_mapping(int base, int size)
     return -1;
 }
 
-
 /* cf. vga_waitretrace, M.Weller */
 int vga_setlinearaddressing(void)
 {
@@ -230,6 +199,8 @@ int vga_setlinearaddressing(void)
 		  + 0xFFF) & 0xFFFFF000;	/* Round up 4K. */
     mappedMemory = memory;
 
+    if (!(modeinfo->flags&CAPABLE_LINEAR)) return -1;
+
     if (lfn) {
 	/* 'Linear' driver function available. */
 	unsigned long mapaddr;
@@ -273,12 +244,17 @@ int vga_setlinearaddressing(void)
       foundlinearframebuffer:
 	(*lfn) (LINEAR_ENABLE, mapaddr);
 	__svgalib_linearframebuffer = map_framebuffer(mapaddr, mappedMemory);
+
 	if ((long) __svgalib_linearframebuffer == -1) {
 	    /* Shouldn't happen. */
 	    (*lfn) (LINEAR_DISABLE, mapaddr);
 	    return -1;
 	}
 	__svgalib_modeinfo_linearset |= IS_LINEAR;
+#ifdef BACKGROUND /* 0 - I don't remember why. */
+        __svgalib_physaddr = (void *)mapaddr; /* remember physical address */
+	__svgalib_linear_memory_size=mappedMemory;
+#endif
 
 	if (memory != mappedMemory)
 	    printf("svgalib: Warning, card has %dK, only %dK available in linear mode.\n",
@@ -289,7 +265,9 @@ int vga_setlinearaddressing(void)
 	(CAPABLE_LINEAR | EXT_INFO_AVAILABLE)) {
 	if (modeinfo->aperture_size >= modeinfo->memory) {
 	    __svgalib_modeinfo_linearset |= IS_LINEAR;
+            __svgalib_physaddr = 
 	    __svgalib_linearframebuffer = modeinfo->linear_aperture;
+	    __svgalib_linear_memory_size=modeinfo->aperture_size<<10;
 	    return modeinfo->memory;
 	}
     }
@@ -394,15 +372,3 @@ int vga_waitevent(int which, fd_set * in, fd_set * out, fd_set * except,
     return retval;
 }
 
-#if 0
-static void reserved_message(void)
-{
-    printf("svgalib: Function not present in shared library image.\n");
-    printf("         You need a newer version of svgalib.\n");
-}
-
-void vga_reserved22(void)
-{
-    reserved_message();
-}
-#endif
