@@ -95,3 +95,26 @@ int vga_getdefaultmode(void) {
     }
     return -1;	/* Not defined */
 }
+
+/* find the mode with minimal resolution (defined as num of pixels) that
+can display x*y */
+int vga_getoptmode(int x, int y, int colors, int bytesperpixel, int c)
+{
+   int i;
+   int cur,size;
+   
+   cur=-1;
+   size=1<<24;
+   
+   for(i=0;i<GLASTMODE;i++)
+      if( vga_hasmode(i)&&
+         (infotable[i].colors==colors)&&
+         (infotable[i].bytesperpixel==bytesperpixel)&&
+         (infotable[i].xdim>=x)&&
+         (infotable[i].ydim>=y)&&
+         (infotable[i].ydim*infotable[i].xdim<size)) {
+            	size=infotable[i].ydim*infotable[i].xdim;
+                cur=i;
+      };
+   return cur;
+};
