@@ -32,24 +32,11 @@ int __svgalib_green_backup[256];
 
 static int set_lut(int index, int red, int green, int blue)
 {
-    if ((__svgalib_chipset == MACH32) && SVGAMODE(CM)) {
-	/* Actually the same but we are in 8514 mode and the dac
-	   does not respond to the VGA circuitry anymore... */
-	if ((index != current_index) || (setget != 1)) {
-	    port_out(index, PEL8514_IW);
-	    current_index = index + 1;
-	    setget = 1;
-	} else
-	    current_index++;
-	__svgalib_delay();
-	port_out(red, PEL8514_D);
-	__svgalib_delay();
-	port_out(green, PEL8514_D);
-	/* Uh yes.. and I didn't find a way to wait for hsync, so... */
-	__svgalib_delay();
-	port_out(blue, PEL8514_D);
-	return 0;
-    }
+    if (__svgalib_novga) return 1;
+    
+    if (infotable[CM].colors>256) return 1; 
+    /* setting palette when in high color mode could be useful, but it is 
+    not a good idea at the moment */
     
     /* prevents lockups */
     if ((__svgalib_chipset == MACH64)) {
@@ -84,23 +71,7 @@ static int set_lut(int index, int red, int green, int blue)
 
 static int get_lut(int index, int *red, int *green, int *blue)
 {
-    if ((__svgalib_chipset == MACH32) && SVGAMODE(CM)) {
-	/* Actually the same but we are in 8514 mode and the dac
-	   does not respond to the VGA circuitry anymore... */
-	if ((index != current_index) || (setget != 0)) {
-	    port_out(index, PEL8514_IR);
-	    current_index = index + 1;
-	    setget = 0;
-	} else
-	    current_index++;
-	__svgalib_delay();
-	*red = (int) port_in(PEL8514_D);
-	__svgalib_delay();
-	*green = (int) port_in(PEL8514_D);
-	__svgalib_delay();
-	*blue = (int) port_in(PEL8514_D);
-/*      return 0; */
-    }
+    if (__svgalib_novga) return 0;
 
     /* prevents lockups on mach64 */
     if ((__svgalib_chipset == MACH64)) {
@@ -170,26 +141,32 @@ int vga_setpalette(int index, int red, int green, int blue)
 	if (green > 255)
 	    green = 255;
     }
+
 #ifdef BACKGROUND
- if (!vga_oktowrite())
-     {
-      vga_setpalette_bg(index, red, green, blue);
-      __svgalib_is_vt_switching_needed();
-      return(0);
-     }
-    tmp=set_lut(index, red, green, blue);
+    if (!vga_oktowrite()) {
+	vga_setpalette_bg(index, red, green, blue);
+	__svgalib_is_vt_switching_needed();
+	return(0);
+    }
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) {
+        tmp =__svgalib_driverspecs->emul->setpalette(index, red, green, blue);
+    } else {
+	tmp = set_lut(index, red, green, blue);
+    }
     __svgalib_is_vt_switching_needed();
     return(tmp);
-#endif
-#ifndef BACKGROUND
-    return set_lut(index, red, green, blue);
+#else
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->setpalette) {
+        return __svgalib_driverspecs->emul->setpalette(index, red, green, blue);
+    } else {
+	return set_lut(index, red, green, blue);
+    }
 #endif
 }
 
 #ifdef BACKGROUND
 
 static void vga_getpalette_bg(int index, int *red, int *green, int *blue)
-
 {
  unsigned char *tmp;
  /* asume max 256 palette places. */
@@ -213,15 +190,20 @@ int vga_getpalette(int index, int *red, int *green, int *blue)
  __svgalib_dont_switch_vt_yet();
 #endif
 #ifndef BACKGROUND
-    get_lut(index, red, green, blue);
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->getpalette) 
+        __svgalib_driverspecs->emul->getpalette(index, red, green, blue);
+    else get_lut(index, red, green, blue);
 #endif
 #ifdef BACKGROUND
  if (!vga_oktowrite())
      {
       vga_getpalette_bg(index, red, green, blue);
      }
-   else
-     get_lut(index, red, green, blue);
+   else {
+    if (__svgalib_driverspecs->emul && __svgalib_driverspecs->emul->getpalette) 
+        __svgalib_driverspecs->emul->getpalette(index, red, green, blue);
+    else get_lut(index, red, green, blue);
+   }
 #endif
     if (__svgalib_grayscale) {
 	if ((unsigned) index >= sizeof(__svgalib_green_backup) / sizeof(__svgalib_green_backup[0])) {
