@@ -1,9 +1,9 @@
 /*
 
-    3DKIT   version   1.2
+    3DKIT   version   1.3
     High speed 3D graphics and rendering library for Linux.
 
-    Copyright (C) 1996  Paul Sheer   psheer@hertz.mech.wits.ac.za
+    Copyright (C) 1996, 1997  Paul Sheer   psheer@icon.co.za
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -28,11 +28,7 @@ File: 3dinit.h
 
 */
 
-
-typedef struct {
-    double x, y, z;
-} Vec;
-
+#include "quickmath.h"
 
 void TD_initcolor (TD_Surface * surf, int n);
 
