@@ -118,12 +118,13 @@ static inline int muldiv64 (int m1, int m2, int d)
 {
 /* int32 * int32 -> int64 / int32 -> int32 */
     int result;
+    int dummy;
     __asm__ (
 		"imull %%edx\n\t"
-		"idivl %3\n\t"
-  :		"=a" (result)	/* out */
-  :		"a" (m1), "d" (m2), "g" (d)	/* in */
-  :		"ax", "dx"	/* mod */
+		"idivl %4\n\t"
+  :		"=a" (result), "=d"(dummy)	/* out */
+  :		"0" (m1), "1" (m2), "g" (d)	/* in */
+		/***rjr***:		"ax", "dx"*/	/* mod */
 	);
     return result;
 }
