@@ -150,7 +150,7 @@ TD_Solid *PL_init_solid (void)
 
     plane_demo->num_surfaces = n;
 
-    if ((plane_demo->surf = malloc (n * sizeof (TD_Surface))) == NULL)
+    if ((plane_demo->surf = calloc (n , sizeof (TD_Surface))) == NULL)
 	return 0;
 
     for (i = 0; i < n; i++) {
@@ -636,7 +636,6 @@ int main (void)
     }
 
     gl_enableclipping ();
-
     PL_animate (plane, PL_redraw, PL_handle_key);
 
     PL_cleanup (plane);
